package ca.aura.artefacts;

import java.io.File;
import java.util.Iterator;

import ca.aabeka.util.Debug;
import ca.aabeka.util.Vector;


/**
 * 
 * @author Andrew
 *
 */
public class AbstractArtefact implements Artefact {

  private static final Location DEFAULT_LOCATION = new Location();
  private static final Vector DEFAULT_CONTENT = new Vector();

  private String id_;
  private Location location_; 
  private Vector aura_;



  //``````````````````````
  // consturction
  //``````````````````````


  /**
   * 
   * set the id of the artefact
   * 
   */

  public AbstractArtefact(String id) {
    init(id); 
  }
  
  private void init(String id) {
    id_ = id;
    aura_ = new Vector();
    addAura(new AbstractAura(id));  
  }


  //``````````````````````
  // artefact interface
  //``````````````````````

  /**
   * 
   * @see ca.aura.artefact.Artefact#getId()
   * 
   */
  public String getId() {
    return id_;
  }

  /**
   * 
   * @see ca.aura.artefacts.Artefact#getName()
   * 
   */
  public String getName() {
    return self().getName();
  }

  /**
   * 
   * @see ca.aura.artefacts.Artefact#getDescription()
   * 
   */
  public Description getDescription() {
    return self().getDescription();
  }


	/**
   * 
	 * @see ca.aura.artefact.Artefact#getLocation()
   * 
	 */
	public File getFile() {
    return (location_ == null) ? new File("") : new File(location_.getFilename());
	}

  /**
   * 
   * @see ca.aura.artefact.Artefact#getAura()
   * 
   */
  public Iterator getAura() {
    return aura_.iterator();
  }


  /**
   * 
   * @see ca.aura.artefact.Artefact#numberOfAura()
   * 
   */

  public int numberOfAura() {
    return aura_.size();  
  }

  //`````````````````````
  // interface
  //`````````````````````

  /**
   * 
   * set the name
   * 
   */
  
  public void setName(String name) {
    self().setName(name);  
  }

  /**
   * 
   * set the description
   * 
   */
  public void setDescription(Description descr) {
    self().setDescription(descr);
  }



  //``````````````````````
  // object interface
  //``````````````````````

  /**
   * 
   * check for equality
   * 
   */

  public boolean equals(Object obj) {

    if (obj instanceof String) {
      String id = (String)obj;
      boolean isEqual = getId().equals(id);
      return isEqual;  
    }

    if (!(obj instanceof Artefact)) {
      return false;
    }

    Artefact artefact = (Artefact) obj;
    if (!getId().equals(artefact.getId())) {
      return false;
    }

    return true;
  }

  /**
   * 
   * the object as a string
   * 
   */

  public String toString() {
    return id_;
  }



  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * get the artefact as a aura (to access the name and descr
   * 
   */
  
  protected AbstractAura self() {
    return (AbstractAura)aura_.elementAt(0);  
  }  

  /**
   * 
   * set the location of the artefact
   * 
   */
  
  public void setLocation(Location location) {
    location_ = location;
  }

  /**
   * 
   * add a segment to the content
   * 
   */
  
  public void addAura(Aura aura) {
    aura_.add(aura);  
  }

}
