package ca.aura.project;

import java.io.File;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aabeka.xml.XmlStream;
import ca.aura.xml.DepotXmlConverter;
import ca.aura.xml.HistoryXmlConverter;
import ca.aura.xml.ProjectXmlConverter;
import ca.aura.xml.StructureXmlConverter;

/**
 * 
 * @author aforward
 *
 */
public class DefaultProject implements Project {

  private String name_;
  private String projectFilename_;
  private String depotFilename_;
  private String historyFilename_;

  private String relationFilename_;
  private String structureFilename_;

  private Depot depot_;
  private History history_;
  private Structure structure_;
  private Relation relation_;

  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  public DefaultProject() {
    init("","");  
  }

  /**
   * 
   * set the name
   * 
   */
  public DefaultProject(String name) {
    init(name, "");  
  }

  /**
   * 
   * set the name
   * 
   */
  public DefaultProject(String name, String filename) {
    init(name, filename);  
  }

  
  
  private void init(String name, String filename) {
    name_ = name;  
    projectFilename_ = filename;
  }


  /**
   * @see ca.aura.project.Project#load()
   */
  public boolean load() {
    
    File projectFile = new File(getFilename());
    
    if (!projectFile.exists() || !projectFile.isFile()) {
      return false;
    }
    
    XmlStream stream = new XmlStream(new ProjectXmlConverter(), projectFile);
    
    if (!stream.read()) {
      return false;  
    }
    
    Project project = (Project)stream.getObject();    
    
    
    setDepotFile(project.getDepotFile());
    setHistoryFile(project.getHistoryFile());
    setRelationFile(project.getRelationFile());
    setStructureFile(project.getStructureFile());
    
    File dir = new File(projectFile.getParent());
    File file;
    
    file = new File(dir,getDepotFile());
    stream = new XmlStream(new DepotXmlConverter(),file);
    if (!stream.read()) {
      return false;  
    }

    depot_ = (Depot)build(dir,getDepotFile(),new DepotXmlConverter());
    history_ = (History)build(dir,getHistoryFile(),new HistoryXmlConverter());
    history_.setDepot(depot_);

    structure_ = (Structure)build(dir,getStructureFile(),new StructureXmlConverter());
    //relation_ = (Relation)build(dir,getRelationFile(),new RelationXmlConverter());


    return true;    
  }



  /**
   * @see ca.aura.project.Project#getDepot()
   */
  public Depot getDepot() {
    return depot_;
  }

  /**
   * @see ca.aura.project.Project#getHistory()
   */
  public History getHistory() {
    return history_;
  }


  /**
   * @see ca.aura.project.Project#getStructure()
   */
  public Structure getStructure() {
    return structure_;
  }

  /**
   * @see ca.aura.project.Project#getRelation()
   */
  public Relation getRelation() {
    return relation_;
  }



  /**
   * 
   * get the project name
   * 
   */  
  public String getName() {
    return name_;  
  }

  /**
   * 
   * get the project file namename
   * 
   */  
  public String getFilename() {
    return projectFilename_;  
  }

  /**
   * 
   * get the depot name
   * 
   */  

  public String getDepotFile() {
    return depotFilename_;  
  }


  /**
   * 
   * get the depot name
   * 
   */  

  public String getHistoryFile() {
    return historyFilename_;  
  }

  /**
   * 
   * get the depot name
   * 
   */  

  public String getStructureFile() {
    return structureFilename_;  
  }


  /**
   * 
   * get the relation name
   * 
   */  

  public String getRelationFile() {
    return relationFilename_;  
  }


  //`````````````````````
  // interface
  //`````````````````````

  /**
   * 
   * set the project name
   * 
   */  
  public void setName(String name) {
    name_ = name;  
  }

  /**
   * 
   * set the project file name
   * 
   */  
  public void setFilename(String projectFilename) {
    projectFilename_ = projectFilename;  
  }

  /**
   * 
   * set the depot name
   * 
   */  

  public void setDepotFile(String filename) {
    depotFilename_ = filename;  
  }



  /**
   * 
   * set the depot name
   * 
   */  

  public void setStructureFile(String filename) {
    structureFilename_ = filename;  
  }


  /**
   * 
   * set the depot name
   * 
   */  

  public void setRelationFile(String filename) {
    relationFilename_ = filename;  
  }

  /**
   * 
   * set the depot name
   * 
   */  

  public void setHistoryFile(String filename) {
    historyFilename_ = filename;  
  }


  //``````````````````````
  // object interface
  //``````````````````````

  /**
   * 
   * check for equality
   * 
   */

  public boolean equals(Object obj) {

    if (!(obj instanceof Project)) {
      return false;
    }

    Project project = (Project) obj;

    if (!toString().equals(project.toString())) {
      return false;
    }

    return true;
  }

  /**
   * 
   * the object as a string
   * 
   */

  public String toString() {
    return name_;
  }
  


  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * build the project elements
   * 
   */

  protected Object build(File dir, String name, ObjectXmlConverter converter) {
    File file = new File(dir,name);
    XmlStream stream = new XmlStream(converter,file);
    stream.read();
    return stream.getObject();    
  }  


}
