package ca.aura.ui.swt;

import java.util.Iterator;
import java.util.Vector;

import ca.aura.ui.resources.RelevancerSettingsResource;
import ca.aura.ui.swt.actions.*;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;


/**
 * 
 * @author Andrew
 *
 */
public class ParametersUI {

  private static int MAX = 20;
  private static int MIN = 0;

  Vector allValues_;


  Label intro_;

  Label ageText_;
  Label integrityText_;
  Label authorityText_;
  Label usageText_;
  Label feedbackText_;
 
  Scale ageScale_;
  Scale integrityScale_;
  Scale authorityScale_;
  Scale usageScale_;
  Scale feedbackScale_;

  

  //``````````````````````
  // constructor
  //``````````````````````

  /**
   * 
   * default
   * 
   */
  
  protected ParametersUI(Composite composite) {
    init(composite);  
  }

  private void init(Composite composite) {
    GridLayout gridLayout;
    GridData gridData;    

    Group  group = new Group(composite,SWT.SHADOW_ETCHED_IN);
    
    group.setText(resource().getGroupText());
    group.setToolTipText(resource().getIntroText());
    group.setBounds(10,10,100,100);

    gridLayout = new GridLayout();
    gridLayout.numColumns = 1;

    gridData = new GridData(GridData.FILL_BOTH);
    gridData.widthHint = 200;
    gridData.heightHint = 50;

    group.setLayout(gridLayout);
    group.setLayoutData(gridData);    
    
    intro_ = new Label(group,SWT.WRAP);
    intro_.setText(resource().getIntroText());
    gridData = new GridData(GridData.FILL_BOTH);
    gridData.widthHint = 200;
    gridData.heightHint = 50;
    intro_.setLayoutData(gridData);


    
    Composite scales = new Composite(group,SWT.NONE);
    gridLayout = new GridLayout();
    gridLayout.numColumns = 5;
    gridData = new GridData(GridData.FILL_BOTH);
    gridData.widthHint = 300;
    gridData.heightHint = 200;    
    gridData.verticalAlignment = GridData.VERTICAL_ALIGN_BEGINNING;
    scales.setLayout(gridLayout);
    scales.setLayoutData(gridData);   

    ageText_ = new Label(scales, SWT.CENTER);    
    integrityText_ = new Label(scales, SWT.CENTER);    
    authorityText_ = new Label(scales, SWT.CENTER);    
    usageText_ = new Label(scales, SWT.CENTER);    
    feedbackText_ = new Label(scales, SWT.CENTER);        

    ageScale_ = new Scale(scales,SWT.VERTICAL);
    integrityScale_ = new Scale(scales,SWT.VERTICAL);
    authorityScale_ = new Scale(scales,SWT.VERTICAL);
    usageScale_ = new Scale(scales,SWT.VERTICAL);
    feedbackScale_ = new Scale(scales,SWT.VERTICAL);

    buildText(ageText_,resource().getAgeText(), resource().getAgeToolTip());
    buildText(integrityText_,resource().getIntegrityText(), resource().getIntegrityToolTip());
    buildText(authorityText_,resource().getAuthorityText(), resource().getAuthorityToolTip());
    buildText(usageText_,resource().getUsageText(), resource().getUsageToolTip());
    buildText(feedbackText_,resource().getFeedbackText(), resource().getFeedbackToolTip());    
   
    buildScale(ageScale_);
    buildScale(integrityScale_);
    buildScale(authorityScale_);
    buildScale(usageScale_);
    buildScale(feedbackScale_);    

    
    MovingControlListener moving = new MovingControlListener();
    group.addMouseListener(moving);
    group.addMouseMoveListener(moving);

  }


  /**
   * 
   * build the Maintenance Table
   * 
   */
  
  public static ParametersUI build(Composite composite) {
    return new ParametersUI(composite);
  }


  /**
   * 
   * build the text name for the scales
   * 
   */
  
  protected void buildText(Label label, String text, String tooltip) {
    label.setText(text);
    label.setToolTipText(tooltip);
    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
    label.setLayoutData(gridData);    
  }

  protected void buildScale(Scale scale) {
    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
    scale.setLayoutData(gridData);    
  }

  //``````````````````````
  // static
  //``````````````````````



  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * get the resource
   * 
   */

  protected RelevancerSettingsResource resource() {
    return RelevancerSettingsResource.getDefault();  
  }


  /**
   * 
   * how many scales can we change?
   * 
   */

  protected int numberOfAvailableScales() {
    int num = 0;
    Iterator iter = allValues_.iterator();
    while (iter.hasNext()) {
      Text value = (Text)iter.next();
      if (value.isEnabled()) {
        num++;  
      }    
    }      
    return num++; 
  }

  /**
   * 
   * how much do we have to change the scales by?
   * 
   */

  protected int valueToChangeScalesBy() {
    
    int count = 0;
    
    Iterator iter = allValues_.iterator();
    while (iter.hasNext()) {
      Text value = (Text)iter.next();
      count += getValue(value.getText());
    }      
    return MAX - count;     
  }

  /**
   * 
   * get the value from a String
   * 
   */

  protected double getValue(String valueString) {
    
    try {
      double value = new Double(valueString).doubleValue();
      if (value > MAX || value < MIN) {
        return MIN;  
      }    
      return value;
    } catch (NumberFormatException e) {
      return MIN;  
    }
  }

}
