package ca.aura.ui.swt;

import java.util.Iterator;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;


import ca.aabeka.util.Vector;
import ca.aura.algorithms.LRFU;
import ca.aura.algorithms.TextSearch;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.reports.LRFUReport;
import ca.aura.artefacts.reports.Report;
import ca.aura.artefacts.sorting.Data;
import ca.aura.artefacts.sorting.ResultSet;
import ca.aura.project.Depot;
import ca.aura.project.History;
import ca.aura.ui.resources.ArtefactResource;
import ca.aura.ui.resources.ResultSetViewResource;
import ca.aura.ui.swt.actions.ActionSelectionListener;
import ca.aura.ui.swt.actions.FileDropTarget;
import ca.aura.ui.swt.actions.LaunchFileAction;
import ca.aura.ui.swt.actions.UnSelectRowListener;

/**
 * 
 * @author aforward
 *
 */
public class ResultSetView {

  private Depot depot_;  
  private History history_;
  private TextSearch ts_;

  private LRFU lrfu_;


  private Composite self_;
  
  private Label label_;
  private Text text_;
  private Combo tasks_;
  private Button go_;
  
  private MenuItem browse_;
  
  private TableViewer viewer_;

  private Vector selectedArtefacts_;

  //``````````````````````
  // constructor
  //``````````````````````
  
  /**
   * 
   * default
   * 
   */
  
  protected ResultSetView(Composite self) {
    init(self);  
  }
  
  private void init(Composite self) {

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.verticalSpacing = 2;
    gridLayout.marginWidth = 0;
    gridLayout.marginHeight = 2;

    self.setLayout(gridLayout);
    
    label_ = buildSearchLabel(self);
    text_ = buildSearchArea(self); 
    //tasks_ = buildTaskCombo(self);
    go_ = buildSearchGo(self);

    viewer_ = new TableViewer(self, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION);
    viewer_.setContentProvider(new ResultSetContentProvider());
    viewer_.setLabelProvider(new ResultSetLabelProvider());
    viewer_.setUseHashlookup(true);

    GridData layoutData = new GridData();
    layoutData.horizontalSpan = 5;
    layoutData.grabExcessHorizontalSpace = true;
    layoutData.grabExcessVerticalSpace = true;
    layoutData.horizontalAlignment = GridData.FILL;
    layoutData.verticalAlignment = GridData.FILL;        

    viewer_.getControl().setLayoutData(layoutData);
    
    FileDropTarget.buildDragAndDrop(viewer_.getControl());


    Table table = viewer_.getTable();

    ArtefactResource resource = ArtefactResource.getDefault();
    Menu menu = new Menu(table.getShell(), SWT.POP_UP);

    browse_ = new MenuItem(menu, SWT.CASCADE);
    browse_.setText(resource.getBrowseMenuItemText());

    MenuItem search = new MenuItem(menu, SWT.CASCADE);
    search.setText(resource.getSearchMenuItemText());
    search.setEnabled(false);
    table.setMenu(menu);    

    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    table.addMouseListener(new UnSelectRowListener());
    table.addSelectionListener(new SelectedArtefact(browse_));
    
    TableColumn column;

    column = new TableColumn(table, SWT.NONE, 0);
    column.setText(resource().getMaintenanceText());
    column.setWidth(100);
    column.pack();

    column = new TableColumn(table, SWT.NONE, 1);
    column.setText(resource().getRankText());
    column.setWidth(100);
    column.pack();

    column = new TableColumn(table, SWT.NONE, 2);
    column.setText(resource().getNameText());
    column.setWidth(100);
    column.pack();

    column = new TableColumn(table, SWT.NONE, 3);
    column.setText(resource().getDescriptionText());
    column.setWidth(100);
    column.pack();


    
    /*
    
    Menu menu = new Menu(viewer_.getTree().getShell(),SWT.POP_UP);
    MenuItem item = new MenuItem(menu,SWT.NULL);
    item.setText("1");
    
    item = new MenuItem(menu,SWT.NULL);
    item.setText("2"); 
    
    viewer_.getTree().setMenu(menu);
    
    */
    
    
    
    
  }

  /**
   * 
   * build a whiteboard ui
   * 
   */  
 
  public static  synchronized ResultSetView build(Composite composite) {
    Composite self = new Composite(composite, SWT.NO_MERGE_PAINTS);
    return buildSelf(self);
  }

  /**
   * 
   * build self
   * 
   */  

  public static synchronized ResultSetView buildSelf(Composite self) {
    return new ResultSetView(self);  
  }


  //`````````````````````
  // interface
  //`````````````````````

  /**
   * 
   * set the vectors
   * 
   */
  
  public void setDepot(Depot depot) {
    depot_ = depot;   
    ts_ = new TextSearch(depot.getArtefacts()); 
  }

  /**
   * 
   * set the history of the depot
   * 
   */
  
  public void setHistory(History history) {
    history_ = history;
    sort();
  }  

  /**
   * 
   * sort the results
   * 
   */
  
  public void sort(Report report) {
    lrfu_ = new LRFU(report);
    showResults(lrfu_.sortedResults());
  }

  /**
   * 
   * sort all the results
   * 
   */
  
  public void sort() {
    if (history_ == null) {
      return;  
    }
    Report report = new LRFUReport(history_);
    sort(report);    
  }
  

  /**
   * 
   * do the search
   * 
   */

  public void doSearch() {
    String query = text_.getText();
    if ("".equals(query)) {
      sort();  
      return;
    }
    boolean addToResults = true;
    ts_.search(query,addToResults);
    Report report = new LRFUReport(history_,ts_.results());
    sort(report);
  }  

  /**
   * 
   * show results
   * 
   */
  
  public void showResults(Iterator results) {
    ResultSet rs = new ResultSet(results);
    setResultSet(rs);       
  }


  /**
   * 
   * set the project depot
   * 
   */

  protected void setResultSet(ResultSet resultSet) {
    viewer_.setInput(resultSet);  
    viewer_.refresh();
  }

  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * build the gui components
   * 
   */
  
  protected Label buildSearchLabel(Composite parent) {
    Label label = new Label(parent,SWT.SHADOW_NONE);
    label.setText("Search:");
    return label;  
  }
 
  protected Text buildSearchArea(Composite parent) {
    Text text = new Text(parent, SWT.SINGLE);
    GridData gridData = new GridData();
    gridData.widthHint = 150;
    text.setLayoutData(gridData);
    text.addSelectionListener(new DoSearch());
    return text; 
  }

  protected Combo buildTaskCombo(Composite parent) {
    Combo combo = new Combo(parent,SWT.DROP_DOWN);
    combo.add("i'm browsing");
    combo.add("i'm maintain");
    GridData gridData = new GridData();
    gridData.widthHint = 75;
    combo.setLayoutData(gridData);
    return combo; 
  }

  protected Button buildSearchGo(Composite parent) {
    Button b = new Button(parent,SWT.FLAT);
    b.setText("go");
    b.addSelectionListener(new DoSearch());
    return b;  
  }

  /**
   * 
   * get the  resource
   * 
   */

  public ResultSetViewResource resource() {
    return  ResultSetViewResource.getDefault();
  }

  /**
   * 
   * get the selected artefact
   * 
   */
  
  protected Iterator getSelectedArtefacts() {
    return selectedArtefacts_.iterator();  
  }

  //`````````````````````
  // inner classe
  //`````````````````````

  public class DoSearch extends SelectionAdapter {
 
    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent event) {
      doSearch();
    }
  }

  public class SelectedArtefact extends SelectionAdapter {
    
    SelectionListener lastListener_;
    MenuItem browse_;
 
    public SelectedArtefact(MenuItem browse) {
      browse_ = browse;  
    }
 
    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent event) {
      Table table = (Table)event.getSource();
      TableItem[] items = table.getSelection();
      
      if (lastListener_ != null) {
        browse_.removeSelectionListener(lastListener_);
      }
      
      if (items.length == 0) {
        return;
      }
      
      Data data = (Data)items[0].getData();
      
      lastListener_ = new ActionSelectionListener(new LaunchFileAction(data.getData().getFile()));
      browse_.addSelectionListener(lastListener_);      
    }
  }


}
