package ca.aura.ui.swt.actions;

import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Control;


public class MovingControlListener implements MouseListener, MouseMoveListener {

  private static final int RIGHT_MOUSE_CLICK =3;
  private static final int LEFT_MOUSE_CLICK =1;

  private boolean isSelected_;

  private Point lastPoint_;


  //````````````````````````
  // Constructor
  //````````````````````````

  /**
   *  
   * Specify the presenter to select.
   * 
   */

  public MovingControlListener() {
    init();  
  }

  private void init() {
    isSelected_ = false;
    lastPoint_ = new Point(0,0);
  }



  //````````````````````````
  // MouseListener
  //````````````````````````

  /**
   *  
   * The Document can handle two mouseDown events
   * 
   *  The "right" click opens up the document menu.
   *  The "left" click picks document to be moved around 
   * 
   */

  public void mouseDown(MouseEvent e) {
    
    if (!isSelected()) {
      lastPoint_ = new Point(e.x,e.y);
      setSelected(true);
    }
  }



  /**
   *  
   * The document is opened when double clicked
   * 
   */

  public void mouseDoubleClick(MouseEvent e) { 

   }
  
  public void mouseUp(MouseEvent e) {
    lastPoint_ = new Point(e.x,e.y);
    setSelected(false);
  }

  //``````````````
  // MouseMoveListener
  //``````````````

  /**
   *  
   * If the mouse is selected (based on a mouseDown event), 
   * then the document is
   *    + moved to the new location
   *    + visually marked as being moved
   * 
   */
  
  public void mouseMove(MouseEvent e) {

    if (!isSelected()) {
      return;
    }

    Control item = (Control)e.getSource();
    
    Rectangle bound = item.getBounds();
    int xDiff = e.x - lastPoint_.x;
    int yDiff = e.y - lastPoint_.y;

    int newX = bound.x + xDiff;
    int newY = bound.y + yDiff;
    //lastPoint_ = new Point(newX, newY);
    item.setLocation(newX,newY);
  }


  //``````````````````````
  // protected
  //``````````````````````

  /**
   *  
   * Is the document currently selected?
   * 
   */
  
  protected boolean isSelected() {
    return isSelected_;  
  }

  /**
   *  
   * Set the document as selected or not
   * 
   */

  protected void setSelected(boolean isSelected) {
    isSelected_ = isSelected;  
  }

  
}
