package ca.aura.ui.swt.actions;

import java.util.Iterator;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;

import ca.aabeka.util.Debug;
import ca.aabeka.util.Graph;
import ca.aura.ui.swt.WhiteBoardView;

/**
 * 
 * @author aforward
 *
 */
public class WhiteBoardEngine {

  WhiteBoardView whiteboard_;
  Graph graph_;
  
  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */

  public WhiteBoardEngine(WhiteBoardView wb, Graph g) {
    init(wb,g);  
  }

  private void init(WhiteBoardView wb, Graph g) {
    whiteboard_ = wb;
    graph_ = g;  
  }

  public void layout(Object obj) {


    Control control = whiteboard_.buildArtefact(obj);
    Point midPoint = getMidPoint(whiteboard_.getControl()); 
    Point dimensions = new Point(control.getBounds().width,control.getBounds().height); 

    control.setBounds(getBounds(midPoint ,dimensions));

    int length = 50;    
    double angle = 0;
    
    double theta = getRotationDegree(obj,graph_);
    Iterator iter = graph_.outReferencesFrom(obj);
    while (iter.hasNext()) {
      Object next = iter.next();
      
      control = whiteboard_.buildArtefact(next);
      
      Rectangle bounds = getBounds(rotate(midPoint,angle,length),dimensions);
      Debug.defaultPrintln(this,"inside:" + bounds);
      control.setBounds(bounds); 
      
      angle -= theta;  
    }
        
    
  }



  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * get the rotation degree
   * 
   */
  
  protected double getRotationDegree(Object obj, Graph g) {
    if (g.outDegreeOf(obj) == 0) {
      return 0;  
    }
    
    return Math.PI*1.9 / (double)g.outDegreeOf(obj);  
    
    
  }

  /**
   * 
   * get the bounds based on the midpoint and the dimennsions of an object
   * 
   */

  protected Rectangle getBounds(Point midPoint, Point dimensions) {
    
    Debug.defaultPrintln(this, midPoint);
    Debug.defaultPrintln(this, dimensions);
    
    return new Rectangle(midPoint.x - dimensions.x/2, midPoint.y + dimensions.y/2, dimensions.x, dimensions.y);
  }
  

  /**
   * 
   * get the midpoint of a control
   * 
   */

  protected Point getMidPoint(Control control) {
    return new Point(control.getBounds().width/2,control.getBounds().height/2);  
  }
  

  /**
   * 
   * rotate a point
   * 
   */

  protected Point rotate(Point midpoint, double radian, double radius) {
    
    double slope = Math.abs(Math.tan(radian));
    
    double epsilon = radius*0.05;
    double x = midpoint.x;
    double y = midpoint.y;
    double increment = 10;
    
    double difference = radius;
    int count=0;
    while ( Math.abs(difference) > epsilon && count < 50) {
      count++;
      
      
      if (difference > epsilon && increment < 0) {
        increment /= -2;  
      }
      
      if (difference < -epsilon && increment > 0) {
        increment /= -2;  
      }

      x = offsetX(x,radian,increment);  
      y = offsetY(y,radian,increment*slope);
      
     difference = radius - length(midpoint,x,y);      
     
     Debug.defaultPrintln(this, "difference: " + difference);

    }
    
    
      
    return new Point((int)x,(int)y);
  }  
  

  /**
   * 
   * length between two points
   * 
   */

  protected double length(Point p1, double x2, double y2) {
    double x1 = (double)p1.x;
    double y1 = (double)p1.y;
    double length = Math.sqrt((Math.pow(x2-x1,2) + Math.pow(y2-y1,2)));
    return Math.abs(length);
  }

  /**
   * 
   * get the x offset
   * 
   */

  protected double offsetX(double x, double rad, double amount) {
    boolean four = Math.cos(rad) > 0 && Math.sin(rad) < 0;
    boolean three = Math.tan(rad) > 0 && Math.cos(rad) < 0;
    boolean two = Math.sin(rad) > 0 && Math.cos(rad) < 0;
    boolean one = !four && !three && !two;
    
    if (one || four) {
      return x + amount;     
    }
    
    return x - amount;
    
  }

  /**
   * 
   * get the y offset
   * 
   */
  
  protected double offsetY(double y, double rad, double amount) {
    boolean four = Math.cos(rad) > 0 && Math.sin(rad) < 0;
    boolean three = Math.tan(rad) > 0 && Math.cos(rad) < 0;
    boolean two = Math.sin(rad) > 0 && Math.cos(rad) < 0;
    boolean one = !four && !three && !two;
    
    if (one || two) {
      return y + amount;     
    }
    
    return y - amount;
    
  }


}
