package ca.aura.xml;

import java.util.Iterator;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.util.DateAndTime;
import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.usage.DefaultUsage;
import ca.aura.artefacts.usage.Usage;
import ca.aura.project.History;

/**
 * 
 * @author aforward
 *
 */
public class HistoryXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_HISTORY = "history"; 

  private static final String ELEMENT_USAGE = "usage";

  private static final String ATTRIBUTE_ARTEFACT = "artefact";  
  private static final String ATTRIBUTE_TASK = "task";
  private static final String ATTRIBUTE_ROLE = "role";
  private static final String ATTRIBUTE_DATE = "date";
  

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_HISTORY;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof History);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {

   
    History history = new History();
    
    if (!canLoad(ele)) {
      return history;
    }

    
    Iterator usages = ele.getChildren(ELEMENT_USAGE).iterator();

    while (usages.hasNext()) {
      history.add(loadUsage((Element)usages.next()));  
    }    
    
    return history;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {

    Element ele = new Element(getElementName());
    
    if (!canBuild(obj)) {
      return ele;  
    }

    History history = (History)obj;
    
    Iterator iter = history.getHistory();
    while (iter.hasNext()) {
      Usage usage = (Usage)iter.next();
      Element usageElement = new Element(ELEMENT_USAGE);  
      usageElement.setAttribute(this.ATTRIBUTE_ARTEFACT,usage.getArtefactId());
      usageElement.setAttribute(this.ATTRIBUTE_TASK, usage.getTask());
      usageElement.setAttribute(this.ATTRIBUTE_ROLE, usage.getRole());
      usageElement.setAttribute(this.ATTRIBUTE_DATE, DateAndTime.getDefault().getTime(usage.getDate()));
      usageElement.setText(usage.getComment());  
      ele.addContent(usageElement);
    }

    return ele;
  }

  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * descr
   * 
   */

  public boolean canLoadUsage(Element ele) {
    return ELEMENT_USAGE.equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Usage loadUsage(Element ele) {

    DefaultUsage usage = new DefaultUsage();
    
    if (!canLoadUsage(ele)) {
      return usage;
    }
    
    Attribute attr;
    
    attr = ele.getAttribute(ATTRIBUTE_ARTEFACT);
    if (attr != null) {
      usage.setArtefactId(attr.getValue());  
    }

    attr = ele.getAttribute(ATTRIBUTE_TASK);
    if (attr != null) {
      usage.setTask(attr.getValue());  
    }

    attr = ele.getAttribute(ATTRIBUTE_ROLE);
    if (attr != null) {
      usage.setRole(attr.getValue());  
    }

    attr = ele.getAttribute(ATTRIBUTE_TASK);
    if (attr != null) {
      usage.setDate(attr.getValue());  
    }
    return usage;
  }


}
