package ca.aura.xml;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefact.Artefact;
import ca.aura.artefact.Reference;

/**
 * 
 * @author Andrew
 *
 */
public class ReferenceXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_ARTEFACT = ArtefactXmlConverter.ELEMENT_ROOT;  

  private static final String ATTRIBUTE_ID = "id";
  private static final String ATTRIBUTE_NAME = "name";


  public static final String ELEMENT_ROOT = ELEMENT_ARTEFACT;

  private Artefact source_;


  //``````````````````````
  // consturctor
  //``````````````````````

  public ReferenceXmlConverter(Artefact artefact) {
      
  }


  /**
   * 
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   * 
   * get the Element name
   * 
   */

  public String getElementName() {
    return ELEMENT_ROOT;  
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   * 
   * <reference name="the name" id="1231" />
   *   <data><artefact id='1'></data>
   * </reference>
   * 
   */
  public Object load(Element ele) {
    
    if (ele == null) {
      return null;	
    }

    if (!ELEMENT_ARTEFACT.equals(ele.getName())) {
      return null;  
    }    

    Attribute id = ele.getAttribute(ATTRIBUTE_ID);
    Attribute name = ele.getAttribute(ATTRIBUTE_ID);

    String idValue = (id != null) ? id.getValue() : "";
    String nameValue = (name != null) ? name.getValue() : "";
    
    return new AbstractReference(idValue, nameValue);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {
    
    Element referenceElement = new Element(ELEMENT_ARTEFACT);  
    
    if (!(obj instanceof Reference)) {
      return referenceElement;
    }
    
    Reference reference = (Reference) obj;
    Attribute name = new Attribute(ATTRIBUTE_ID,reference.getId());
    Attribute id = new Attribute(ATTRIBUTE_NAME, reference.getName());
    referenceElement.setAttribute(name);
    referenceElement.setAttribute(id);

    return referenceElement;
  }

}
