package ca.aura.algorithms;

import ca.aabeka.util.DateAndTime;
import ca.aabeka.util.Hashtable;
import ca.aura.algorithms.lrfu.F;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.usage.DefaultUsage;
import ca.aura.artefacts.usage.Usage;
import ca.aura.artefacts.reports.DefaultReport;
import ca.aura.artefacts.reports.Report;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * 
 * @author aforward
 *
 */
public class LRFUTest extends TestCase {

  //```````````````````
  // all tests
  //```````````````````

  /**
   * 
   * the main runner
   * 
   */

  public static void main() {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * 
   * the test suite to group tests together
   * 
   */

  public static Test suite() {
    return new TestSuite(LRFUTest.class);
  }

  //```````````````````
  // setup
  //```````````````````

  /**
   * 
   * create the tmp artefacts
   * 
   */
  public class TmpArtefact extends AbstractArtefact {
  
    public TmpArtefact(int id) {
      super(id + "");
    }   
  }

  /**
   * 
   * create a simple F calculator
   * 
   */
  public class TmpF extends F {

    public int p() {
      return 2;  
    }  
  
    public double lambda() {
      return 1;  
    }
  
  }

  //```````````````````
  // assertions
  //```````````````````

  /**
   * 
   * assert getResults
   * 
   */
  public void assert_getResults(Report report, F f,Hashtable expected) {
    assertTrue("the f function should be valid",f.isValid());
    LRFU lrfu = new LRFU(report, f);
    Hashtable actual = lrfu.getResults();
    assertEquals(expected,actual);  
  }

  /**
   * 
   * assert getResults
   * 
   */
  public void assert_getResults(Report report, Hashtable expected) {
    LRFU lrfu = new LRFU(report);
    Hashtable actual = lrfu.getResults();
    assertEquals(expected,actual);  
  }


  //```````````````````
  // tests
  //```````````````````

  /**
   * 
   * getResults
   * 
   */

  public void test_getResults_nothing() {
    Report report = new DefaultReport();
    Hashtable expected = new Hashtable();
    assert_getResults(report,expected);  
  }

  public void test_getResults_createMyOwnReport() {
    DefaultReport report = new DefaultReport(DateAndTime.add(-1));
    
    Artefact artefact;
    Usage usage;
    
    
    artefact = new TmpArtefact(1);
    usage = new DefaultUsage(artefact.getId(),DateAndTime.add(-6));
    report.add(artefact, usage);

    usage = new DefaultUsage(artefact.getId(),DateAndTime.add(-4));
    report.add(artefact, usage);
    
    usage = new DefaultUsage(artefact.getId(),DateAndTime.add(-3));
    report.add(artefact, usage);    
    
    
    Hashtable expected = new Hashtable();
    expected.put(artefact,new Double(13.0/32.0));

    assert_getResults(report,new TmpF(),expected);  
  }


}
