package ca.aura.algorithms.normalizer;

import ca.aabeka.math.Sequence;
import ca.aura.algorithms.rank.Rank;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * 
 * @author Andrew
 *
 */
public class IncreasingDropOffRankNormalizerTest extends AbstractRankNormalizerTest {


  //```````````````````
  // all tests
  //```````````````````
  
  /**
   * 
   * the main runner
   * 
   */
  
  public static void main () {
    junit.textui.TestRunner.run (suite());
  }
  
  /**
   * 
   * the test suite to group tests together
   * 
   */
  
  public static Test suite() {
    return new TestSuite(IncreasingDropOffRankNormalizerTest.class);
  }


  //```````````````````
  // abstract interface
  //```````````````````

  /**
   * 
   * getRank (drop off curve)
   * 
   * mean = 4
   * st dev = 2
   * 
   * rank 1, value < 0
   * rank 2, value < 2
   * rank 3, value < 4
   * rank 4, value < 6
   * rank 2, value >= 6
   * 
   */

  public RankNormalizer getRankNormalizer(Sequence sequence) {
    return new IncreasingDropOffRankNormalizer(sequence);    
  }
  
  public double getCoef(int rank) {
    if (rank == Rank.getUnknown()) {
      return 3;  
    } else if (rank == 1) {
      return -0.1;
    } else if (rank == 2) {
      return 1.9;
    } else if (rank == 3) {
      return 3.9;
    } else if (rank == 4) {
      return 5.9;
    } else {
      fail("unknown rank provided");
      return -1; 
    }
      
  }

  //```````````````````
  // more tests
  //```````````````````

  public void test_getRank_2b() {
    assert_getRank(getRankNormalizer(buildSequence()),2,6);
  }

}
