package ca.aura.ui.swt;

import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import ca.aabeka.util.Debug;
import ca.aabeka.util.DefaultGraph;
import ca.aabeka.util.Graph;
import ca.aura.ui.resources.ArtefactResource;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * 
 * @author aforward
 *
 */
public class GraphUITestUI extends AbstractTestUI {


  //```````````````````
  // all tests
  //```````````````````

  /**
   * 
   * the main runner
   * 
   */

  public static void main() {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * 
   * the test suite to group tests together
   * 
   */

  public static Test suite() {
    return new TestSuite(GraphUITestUI.class);
  }


  //`````````````````````
  // tests
  //`````````````````````

  /**
   * 
   * test the creation of the graph
   * 
   */
  
  public void test_constructor() {

    getShell().setLayout(new FillLayout());
    
    Canvas board = new Canvas(getShell(), SWT.NULL);
    Rectangle b = getShell().getBounds();
    board.setBounds(b);
    board.setBackground(new Color(null,10,220,10));
    
    
    Debug.defaultPrintln(this,board.getBounds());

    Image img = ArtefactResource.getDefault().getDocImageDescriptor().createImage();
    
    Point dimensions = new Point(img.getBounds().width, img.getBounds().height);
    Point midPoint = getMidPoint(board);
    int length = 50;    
        
    Graph g = new DefaultGraph();
    g.add("a","b");
    g.add("a","c");
    g.add("a","c");
    g.add("a","d");    
    g.add("a","e");
    g.add("a","f");

    
    Label box = new Label(board, SWT.NULL);
    box.setImage(img);
    box.setBounds(getBounds(midPoint ,dimensions)); 

    Debug.defaultPrintln(this,"box:" + box.getBounds());

    double theta = Math.PI*1.9 / (double)g.outDegreeOf("a");
    
    Debug.defaultPrintln(this,"theta: " + theta);
    
    
    double angle = 0;
    
    Debug.defaultPrintln(this,g.outDegreeOf("a"));
    
    Iterator iter = g.outReferencesFrom("a");
    while (iter.hasNext()) {
      iter.next();
      Label ref = new Label(board, SWT.NULL);
      ref.setImage(img);
      Rectangle bounds = getBounds(rotate(midPoint,angle,length),dimensions);
      Debug.defaultPrintln(this,"inside:" + bounds);
      ref.setBounds(bounds); 
      angle -= theta;  
    }
    
    
    assert_gui("Testing Label Provider","Were they okay?"); 
    
    
  }


  protected Rectangle getBounds(Point midPoint, Point dimensions) {
    
    Debug.defaultPrintln(this, midPoint);
    Debug.defaultPrintln(this, dimensions);
    
    return new Rectangle(midPoint.x - dimensions.x/2, midPoint.y + dimensions.y/2, dimensions.x, dimensions.y);
  }

  protected Point getMidPoint(Control control) {
    return new Point(control.getBounds().width/2,control.getBounds().height/2);  
  }

  protected Point rotate(Point midpoint, double radian, double radius) {
    
    double slope = Math.abs(Math.tan(radian));
    
    double epsilon = radius*0.05;
    double x = midpoint.x;
    double y = midpoint.y;
    double increment = 10;
    
    double difference = radius;
    int count=0;
    while ( Math.abs(difference) > epsilon && count < 50) {
      count++;
      
      
      if (difference > epsilon && increment < 0) {
        increment /= -2;  
      }
      
      if (difference < -epsilon && increment > 0) {
        increment /= -2;  
      }

      x = offsetX(x,radian,increment);  
      y = offsetY(y,radian,increment*slope);
      
     difference = radius - length(midpoint,x,y);      
     
     Debug.defaultPrintln(this, "difference: " + difference);

    }
    
    
      
    return new Point((int)x,(int)y);
  }

  protected double length(Point p1, double x2, double y2) {
    double x1 = (double)p1.x;
    double y1 = (double)p1.y;
    double length = Math.sqrt((Math.pow(x2-x1,2) + Math.pow(y2-y1,2)));
    Debug.defaultPrintln(this, "the length: " + length);
    return Math.abs(length);
  }

  protected double offsetX(double x, double rad, double amount) {
    boolean four = Math.cos(rad) > 0 && Math.sin(rad) < 0;
    boolean three = Math.tan(rad) > 0 && Math.cos(rad) < 0;
    boolean two = Math.sin(rad) > 0 && Math.cos(rad) < 0;
    boolean one = !four && !three && !two;
    
    if (one || four) {
      return x + amount;     
    }
    
    return x - amount;
    
  }


  protected double offsetY(double y, double rad, double amount) {
    boolean four = Math.cos(rad) > 0 && Math.sin(rad) < 0;
    boolean three = Math.tan(rad) > 0 && Math.cos(rad) < 0;
    boolean two = Math.sin(rad) > 0 && Math.cos(rad) < 0;
    boolean one = !four && !three && !two;
    
    if (one || two) {
      return y + amount;     
    }
    
    return y - amount;
    
  }

}
