package ca.aura.xml;

import java.io.File;
import java.util.Iterator;

import ca.aabeka.util.Debug;
import ca.aabeka.util.PathContent;
import ca.aabeka.util.Vector;
import ca.aabeka.xml.XmlStream;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Doc;
import ca.aura.artefacts.Location;
import ca.aura.artefacts.usage.DefaultUsage;
import ca.aura.project.Depot;
import ca.aura.project.History;
import ca.aura.project.Id;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * 
 * @author aforward
 *
 */
public class BuildArtefactTest extends TestCase {


  //```````````````````
  // all tests
  //```````````````````

  /**
   * 
   * the main runner
   * 
   */

  public static void main() {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * 
   * the test suite to group tests together
   * 
   */

  public static Test suite() {
    return new TestSuite(BuildArtefactTest.class);
  }

  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * add the files
   * 
   */
  
  protected void buildFile(Depot depot, Iterator files) {
    while (files.hasNext()) {
      File f = (File)files.next();
      
      AbstractArtefact artefact = new Doc(Id.next());
      artefact.setName(f.getName());
      artefact.setLocation(new Location(f.getAbsolutePath()));
      depot.add(artefact);
    }    
  }

  /**
   * 
   * add the Directories
   * 
   */
  
  protected void buildDir(Depot depot, Iterator dirs) {
    PathContent pc = new PathContent();
    while (dirs.hasNext()) {
      File dir = (File)dirs.next();
      buildFile(depot,pc.getFiles(dir).iterator());
      buildDir(depot,pc.getDirectories(dir).iterator());
    }
    
  }


  //`````````````````````
  // assertions
  //`````````````````````

  /**
   * 
   * assert the building of the xml
   * 
   */

  public void assert_build(File dir, File file) { 
    Depot depot = new Depot();
    
    PathContent pc = new PathContent();
    buildFile(depot,pc.getFiles(dir).iterator());
    buildDir(depot,pc.getDirectories(dir).iterator());    

    DepotXmlConverter dxc = new DepotXmlConverter();

    XmlStream stream = new XmlStream(dxc,file);
    
    
    Vector v = new Vector();
    v.addAll(depot.getArtefacts());
    Debug.defaultPrintln(this, v);    

    assertTrue("should be able to read stream:" + file, stream.write(depot) );

  }


  public void assert_buildHistory(History history, File file) { 
    XmlStream stream = new XmlStream(new HistoryXmlConverter(),file);
    assertTrue("should be able to write to stream:" + file, stream.write(history));  
  }
  
  
  
  //`````````````````````
  // tests
  //`````````````````````

  /**
   * 
   * build the artefact depot
   * 
   */

  public void test_build() {
    File dir = new File("D:\\users\\aforward\\projects\\vcr\\doc");
    File file = new File("D:\\users\\aforward\\projects\\vcr\\depot.txt");
    assert_build(dir,file);      
  }
    
  public void test_build_history() {   
    File file = new File("D:\\users\\aforward\\projects\\vcr\\history.txt");
    
    History history = new History();
    
    
    for (int i=0; i< 5; i++) {
      DefaultUsage usage = new DefaultUsage(i+"");
      usage.setComment("the comment: " + i);
      usage.setRole("developer");
      usage.setTask("searching");
      history.add(usage);
    }
    
    assert_buildHistory(history,file);   
  }


}
