package ca.aura.algorithms.normalizer;

import ca.aabeka.math.NormalDistribution;
import ca.aabeka.math.Sequence;
import ca.aura.algorithms.rank.Rank;

/**
 * @author aforward
 *
 */
public abstract class RankNormalizer {

  private static final String INCREASING_CURVE = "increasing-curve";
  private static final String DROP_OFF_CURVE = "drop-off-curve";
  private static final String DECREAING_CURVE = "increasing-curve";

  private static final String DEFAULT_CURVE = DROP_OFF_CURVE;  
  
  private String type_;
  private Sequence sequence_;
  private NormalDistribution dist_;
  private double[] rankDivisions_;
  private Rank[] ranks_;
 
  
  //`````````````````
  // constructor
  //`````````````````

  /**
   * 
   * give a sequence to turn into a normal distribution
   * 
   */

  public RankNormalizer(Sequence sequence) {
    init(new NormalDistribution(sequence));  
  }


  private void init(NormalDistribution dist) {
    dist_ = dist;
    rankDivisions_ = getRankDivisions();
    ranks_ = getRanks();
  }


  //`````````````````
  // abstract
  //`````````````````
  
  /**
   * 
   * get the divisions of st. dev to break up the rank into sections
   * 
   */
  
  public abstract double[] getRankDivisions();
  
  
  /**
   * 
   * get the ranks for the divisions defined above
   * 
   * if divisions.length = 3 then ranks.length must == 4
   * 
   */
  
  public abstract Rank[] getRanks();
  
  

  //`````````````````
  // interface
  //`````````````````

  /**
   * 
   * get the rank of a particular number
   * 
   */ 

  public Rank getRank(double value) {
    
    if (!dist_.isValid()) {
      return new Rank();	
    } 
    
    if (rankDivisions_.length + 1 != ranks_.length) {
      return new Rank();	
    }
    
    int i;
    for (i=0; i<rankDivisions_.length; i++) {
      if (dist_.isLessThan(value,rankDivisions_[i])) {
        return ranks_[i];	
      }	
    }
    return ranks_[i];
  }

  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * build the normal distribution of the sequence
   * 
   */

  protected NormalDistribution buildNormalDistribution() {
    return new NormalDistribution(sequence_);  
  }

}
