package ca.aura.algorithms.rank;

import java.util.Iterator;

import ca.aabeka.util.Hashtable;

/**
 * 
 * @author Andrew
 *
 */
public class Parameters {

  Hashtable values_;
  
  boolean isChanged_;
  double oldTotal_;
  
  
  //``````````````````````
  // consturctor
  //``````````````````````

  /**
   * 
   * default
   * 
   */

  public Parameters() {
    init(new Hashtable());  
  } 

  /**
   * 
   * init with settings in place
   * 
   */
  
  public Parameters(Hashtable hash) {
    init(hash);  
  }

  private void init(Hashtable hash) {
    values_ = hash;
    isChanged_ = false;
    oldTotal_ = calculateTotalWeight();
  }

  //``````````````````````
  // interface
  //``````````````````````

  /**
   * 
   * get the relevance fraction
   * 
   */
  public double getFraction(String id) {
    return getWeight(id) / getTotalWeight();
  }

  /**
   * 
   * get the relevance weight
   * 
   */

  public double getWeight(String id) {
    if (!values_.containsKey(id)) {
      return 0;   
    }  
    return ((Double)values_.get(id)).doubleValue();
  }

  /**
   * 
   * set the relevance weight
   * 
   */

  public boolean setWeight(String id, int value) {
    if (value < 0) {
      return false;  
    }  
    
    isChanged_ = true;
    values_.put(id,new Double(value));
    return true;
  }  

  //``````````````````````
  // protected
  //``````````````````````
  
  /**
   * 
   * get the total weight
   * 
   */

  protected double getTotalWeight() {
    if (!isChanged_) {
      return oldTotal_;    
    }
    isChanged_ = false;
    oldTotal_ = calculateTotalWeight();
    return oldTotal_;
  }  

  /**
   * 
   * calculate the total weight
   * 
   */

  protected double calculateTotalWeight() {
    double total = 0;
    Iterator iter = values_.getValues();
    while(iter.hasNext()) {
      Double weight = (Double)iter.next();
      total += weight.doubleValue();
    }
    return total;
  }

}
