package ca.aura.artefacts.reports;

import java.util.Date;
import java.util.Iterator;

import ca.aabeka.util.Hashtable;
import ca.aabeka.util.Vector;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.usage.Usage;

/**
 * 
 * @author aforward
 *
 */
public class DefaultReport implements Report {

  private static final Vector EMPTY_VECTOR = new Vector();

  Hashtable hash_;
  Date date_;
  
  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  
  public DefaultReport() {
    init(null);  
  }

  /**
   * 
   * set the date of the report
   * 
   */
  
  public DefaultReport(Date date) {
    init(date);  
  }
  
  private void init(Date date) {
    hash_ = new Hashtable(); 
    date_ = date;
  }

  
  //`````````````````````
  // Report interface
  //`````````````````````

  /**
   * 
   * @see ca.aura.artefacts.usage.Report#getDate()
   * 
   */

  public Date getDate() {
    return (date_ == null) ? new Date() : date_;
  }


  /**
   * 
   * @see ca.aura.artefacts.usage.Report#getArtefacts()
   * 
   */
  public Iterator getArtefacts() {
    return hash_.getKeys();
  }

  /**
   * 
   * @see ca.aura.artefacts.usage.Report#isUsed(Artefact)
   * 
   */
  
  public boolean isUsed(Artefact artefact) {
    return getUsage(artefact).hasNext();
  }

  /**
   * 
   * @see ca.aura.artefacts.usage.Report#getUsage(Artefact)
   * 
   */
  public Iterator getUsage(Artefact artefact) {
    
    if (!hash_.containsKey(artefact)) {
      return EMPTY_VECTOR.iterator();   
    }
    
    return ((Vector)hash_.get(artefact)).iterator();
  }


  /**
   * 
   * @see ca.aura.artefacts.usage.Report#getLastUsage(Artefact)
   * 
   */
  public Usage getLastUsage(Artefact artefact) {
    if (!isUsed(artefact)) {
      return null;  
    }
    
    Vector usage = (Vector)hash_.get(artefact);
    return (Usage)usage.lastElement();
    
  }


  /**
   * 
   * add a usage stat to the artefact
   * 
   */

  public void add(Artefact artefact, Usage usage) {
    add(artefact);
    Vector v = (Vector)hash_.get(artefact);
    insertInto(v,usage);    
  }

  /**
   * 
   * add an artefact without usage
   * 
   */

  public void add(Artefact artefact) {
    if (!hash_.containsKey(artefact)) {
      hash_.put(artefact, new Vector());
    }      
  }

  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * add a usage item
   * 
   */

  protected void insertInto(Vector v, Usage usage) {

    for (int i=0; i < v.size(); i++) {
      Usage tmpUsage = (Usage)v.elementAt(i);

      if (usage.getDate().before(tmpUsage.getDate())) {
        v.insertElementAt(usage, i);  
        return;
      }
    }

    v.addElement(usage);  
  }


}
