package ca.aura.ui.swt;

import ca.aabeka.util.Debug;
import ca.aura.ui.resources.MaintenanceTaskResource;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

/**
 * 
 * @author Andrew
 *
 */
public class MaintenanceTask {

  private Composite composite_;
  private Table table_;

  //``````````````````````
  // constructor
  //``````````````````````

  /**
   * 
   * default
   * 
   */
  
  protected MaintenanceTask(Composite composite) {
    init(composite);  
  }
 
  private void init(Composite composite) {
    composite_ = composite;
    table_ = new Table(composite, SWT.MULTI | SWT.BORDER | SWT.FULL_SELECTION);

    buildLook(table_);
    buildColumns(table_); 

    TableItem item = new TableItem(table_, SWT.NONE);
    item.setText(new String[] { "1","2","3","4" });
    
    item = new TableItem(table_,SWT.NONE);
    item.setText(new String[] {"a","b","c","d"});


    table_.addMouseListener(new MouseAdapter() {
      public void mouseDown(MouseEvent e) {
        
       
        Table table = (Table)e.getSource();
        
        TableItem[] items = table.getSelection();
        if (items.length == 0) {
          table.deselectAll();
          return;
        }
        
        int y = e.y;
        
        Rectangle min = items[0].getBounds(0);
        Rectangle max = items[items.length-1].getBounds(0);
        
        int minY = min.y;
        int maxY = max.y + max.width;
        
        if (y < minY || y > maxY) {
          table.deselectAll();
          Debug.defaultPrintln(this,"unselecting");
          return;  
        }
        
      }
    });


  }


  //``````````````````````
  // interface
  //``````````````````````


  /**
   * 
   * build the Maintenance Table
   * 
   */
  
  public static MaintenanceTask build(Composite composite) {
    return new MaintenanceTask(composite);
  }


  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * set the look of the table
   * 
   */

  protected void buildLook(Table table) {
    table.setEnabled(true);
    table.setHeaderVisible(true);
    table.setLinesVisible(true);
    table.setVisible(true);    
    table.setMenu(buildMenu());    
  }

  /**
   * 
   * get the column names
   * 
   */

  protected String[] getColumnNames() {
    
    
  
    String[] names = { resource().getStatusName(), 
                       resource().getDescriptionName(), 
                       resource().getArtefactName(), 
                       resource().getLocationName() 
                     };
    
    return names;
    
  }

  /**
   * 
   * get the default column widths
   * 
   */

  protected int[] getColumnWidths() {
    
    MaintenanceTaskResource resource = MaintenanceTaskResource.getDefault();
  
    int[] widths = { resource.getStatusWidth(), 
                    resource.getDescriptionWidth(), 
                    resource.getArtefactWidth(), 
                    resource.getLocationWidth() 
                  };
    
    return widths;
    
  }


  /**
   * 
   * set the table columns
   * 
   */


  private void buildColumns(Table table) {
    
    String[] columnNames = getColumnNames();
    int[] columnWidths = getColumnWidths();
    
    for(int i = 0; i < columnNames.length; i++) {
      Debug.defaultPrintln(this,"columm: " + columnNames[i]);
      TableColumn column = new TableColumn(table, SWT.NONE);
      column.setText(columnNames[i]);
      column.setWidth(columnWidths[i]);
      /*
      column.addSelectionListener(new SelectionAdapter() {    
        public void widgetSelected(SelectionEvent e) {
          Debug.defaultPrintln(this,e);
        }
      });
      */
    }    
  }


  private Menu buildMenu() {
    Menu menu = new Menu(composite_.getShell(), SWT.POP_UP);
  
    menu.addMenuListener(new MenuAdapter() {
      public void menuShown(MenuEvent e) {
        
        Menu menu = (Menu)e.widget;
        boolean isEnabled = table_.getSelectionCount() != 0;
        
        Debug.defaultPrintln(this,table_.getSelectionCount());

        MenuItem[] items = menu.getItems();
        for (int i=0; i < items.length; i++) {
          items[i].setEnabled(isEnabled);  
        }
      }
      
      public void mouseDown(MenuEvent e) {
        Debug.defaultPrintln(this, "get source: " +  e.getSource());       
      }

      
    });
  
    MaintenanceTaskResource resource = MaintenanceTaskResource.getDefault();
  
    MenuItem completed = new MenuItem(menu, SWT.CASCADE);
    completed.setText(resource.getCompletedMenuName());

    MenuItem verify = new MenuItem(menu, SWT.CASCADE);
    verify.setText(resource.getNeedsVerificationMenuName());

    MenuItem maintain = new MenuItem(menu, SWT.CASCADE);
    maintain.setText(resource.getNeedsMaintenanceMenuName());

    MenuItem ignore = new MenuItem(menu, SWT.CASCADE);
    ignore.setText(resource.getIgnoreMenuName());

    return menu;
  }

  /**
   * 
   * get the resource
   * 
   */

  protected MaintenanceTaskResource resource() {
    return MaintenanceTaskResource.getDefault();  
  }


}
