package ca.aura.ui.swt;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import ca.aabeka.util.Vector;
import ca.aura.artefacts.Aura;
import ca.aura.artefacts.sorting.Data;
import ca.aura.artefacts.sorting.Folder;
import ca.aura.project.Structure;


/**
 * 
 * @author aforward
 *
 */
public class StructureContentProvider implements ITreeContentProvider {

  private static final Object[] NO_CHILDREN = new Object[]{};

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(Object)
   */
  public Object[] getChildren(Object obj) {
    
    if (obj instanceof Structure) {
      return ((Structure)obj).getAllChildren().toArray();  
    } else if (obj instanceof Folder) {
      return ((Folder)obj).getAllChildren().toArray();
    } else if (obj instanceof Data) {
      Data data = (Data)obj;
      
      if (!data.hasData()) {
        return NO_CHILDREN;  
      } 

      if (data.getData().numberOfAura() == 1) {
        return NO_CHILDREN;   
      }

      Vector children = new Vector();
      children.addAll(data.getData().getAura());
      return children.toArray();
    } else if (obj instanceof Aura) {
      return NO_CHILDREN;  
    }

    return NO_CHILDREN;
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(Object)
   */
  public Object getParent(Object obj) {

    if (obj instanceof Structure) {
      return null;  
    } else if (obj instanceof Folder) {
      return ((Folder)obj).getParent();
    } else if (obj instanceof Data) {
      return ((Data)obj).getParent();
    } else if (obj instanceof Aura) {
      return ((Aura)obj).belongsTo();  
    }

    return null;
  }

  /**
   * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(Object)
   */
  public boolean hasChildren(Object obj) {
    return getChildren(obj).length > 0;
  }

  /**
   * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(Object)
   */
  public Object[] getElements(Object obj) {
    return getChildren(obj);
  }

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#dispose()
   */
  public void dispose() {}

  /**
   * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(Viewer, Object, Object)
   */
  public void inputChanged(Viewer arg0, Object arg1, Object arg2) {}

}
