package ca.aura.ui.swt;

import java.util.Iterator;

import ca.aabeka.util.Debug;
import ca.aabeka.util.Graph;
import ca.aabeka.util.Hashtable;
import ca.aura.algorithms.TextSearch;
import ca.aura.artefacts.Artefact;
import ca.aura.project.Depot;
import ca.aura.ui.resources.WhiteBoardResource;
import ca.aura.ui.swt.actions.*;
import ca.aura.ui.swt.actions.AddArtefactAction;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Text;
/**
 * 
 * @author Andrew
 *
 */
public class WhiteBoardView {

  private Composite self_;
  private Point mousePointer_;

  private Hashtable hash_;

  private Label label_;
  private Text text_;
  private Combo tasks_;
  private Button go_;

  private Canvas area_;

  private Depot depot_;
  private TextSearch ts_;

  //``````````````````````
  // constructor
  //``````````````````````
  
  /**
   * 
   * default
   * 
   */
  
  protected WhiteBoardView(Composite self) {
    init(self);  
  }
  
  private void init(Composite self) {
    
    self_ = self;

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.verticalSpacing = 2;
    gridLayout.marginWidth = 0;
    gridLayout.marginHeight = 2;

    self_.setLayout(gridLayout);

    hash_ = new Hashtable();

    label_ = buildSearchLabel(self);
    text_ = buildSearchArea(self); 
    go_ = buildSearchGo(self);


    mousePointer_ = new Point(0,0); 

    GridData layoutData = new GridData();
    layoutData.horizontalSpan = 5;
    layoutData.grabExcessHorizontalSpace = true;
    layoutData.grabExcessVerticalSpace = true;
    layoutData.horizontalAlignment = GridData.FILL;
    layoutData.verticalAlignment = GridData.FILL; 
    //layoutData.widthHint = 300;
    //layoutData.heightHint = 300;
    
    area_ = new Canvas(self, SWT.NULL);
    area_.setLayoutData(layoutData);
    
    area_.setBackground(resource().getBackgroundColour());
    
    CaretListener caret = new CaretListener(mousePointer_);
    area_.addMouseListener(caret);
    area_.addMouseMoveListener(caret);
    
    buildMenu();
    
    FileDropTarget.buildDragAndDrop(area_);
    
    
    
  }

  /**
   * 
   * build a whiteboard ui
   * 
   */  
 
  public static  synchronized WhiteBoardView build(Composite composite) {
    Canvas self = new Canvas(composite, SWT.NO_MERGE_PAINTS);
    return buildSelf(self);
  }

  /**
   * 
   * build self
   * 
   */  

  public static synchronized WhiteBoardView buildSelf(Composite self) {
    return new WhiteBoardView(self);  
  }


  //``````````````````````
  // interface
  //``````````````````````


  /**
   * 
   * remove all elements
   * 
   */
  
  public void removeAll() {
    Control[] controls = area_.getChildren();
    for (int i=0; i< controls.length; i++) {
      controls[i].dispose();    
    }
  }


  /**
   * 
   * build the graph
   * 
   */

  public void setDepot(Depot depot) {

    removeAll();

    depot_ = depot;   
    ts_ = new TextSearch(depot.getArtefacts()); 
    
    Iterator iter = depot.getArtefacts();
 
    Point point = new Point(10,10);
    
    while (iter.hasNext()) {
      
      Debug.defaultPrintln(this, "the point: " + point);
      if (point.x > 500) {
        point.x = 10;
        point.y = point.y + 60;  
      } else {
        point.x = point.x + 30;
      }
      Artefact artefact = (Artefact)iter.next();
      ArtefactView aui = new ArtefactView(artefact, area_,point.x,point.y);
      aui.normal();
      hash_.put(artefact, aui);
    }  
    
  }

  /**
   * 
   * do the search
   * 
   */

  public void doSearch() {
    ts_.search(text_.getText());
    showResults(ts_.results());    
  } 
  
  /**
   * 
   * set the results
   * 
   */

  public void showResults(Iterator results) {
    
    Iterator reset = hash_.getValues();
    while (reset.hasNext()) {
      ((ArtefactView)reset.next()).normal();  
    }

    while (results.hasNext()) {
      ArtefactView ui = (ArtefactView)hash_.get(results.next());
      ui.highlight();
    }      
    
  }


  /**
   * 
   * build the graph
   * 
   */

  public void setGraph(Object obj, Graph g) {

    Rectangle b = area_.getParent().getBounds();
    area_.setBounds(b);
    WhiteBoardEngine engine = new WhiteBoardEngine(this,g);

    engine.layout(obj);
    
  }

  /**
   * 
   * add a new artefact to the default location
   * 
   */

  public void addNewArtefact() {
    Artefact artefact = null;
    ArtefactView.build(artefact,area_,mousePointer_.x,mousePointer_.y);  
  }

  /**
   * 
   * add an artefact
   * 
   */ 

  public Control buildArtefact(Object obj) {
    Artefact artefact = null;
    ArtefactView aui = ArtefactView.build(artefact, area_,0,0);
    return aui.getControl();
    
  }

  /**
   * 
   * get control
   * 
   */
  
  public Control getControl() {
    return area_;  
  }

  //``````````````````````
  // protected
  //``````````````````````
  
  /**
   * 
   * get the ui resource
   * 
   */  

  protected WhiteBoardResource resource() {
    return WhiteBoardResource.getDefault();  
  }

  /**
   * 
   * build the menu
   * 
   */    

  protected void buildMenu() {
    Menu menu = new Menu(area_.getShell(), SWT.POP_UP);

    MenuItem addComment = new MenuItem(menu, SWT.CASCADE);
    addComment.setText(resource().getAddDocumentText());
    addComment.addSelectionListener(new ActionSelectionListener(new AddArtefactAction(this)));
    area_.setMenu(menu);
  }



  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * build the gui components
   * 
   */
  
  protected Label buildSearchLabel(Composite parent) {
    Label label = new Label(parent,SWT.SHADOW_NONE);
    label.setText("Search:");
    return label;  
  }
 
  protected Text buildSearchArea(Composite parent) {
    Text text = new Text(parent, SWT.SINGLE);
    GridData gridData = new GridData();
    gridData.widthHint = 150;
    text.setLayoutData(gridData);
    text.addSelectionListener(new DoSearch());
    return text; 
  }

  protected Combo buildTaskCombo(Composite parent) {
    Combo combo = new Combo(parent,SWT.DROP_DOWN);
    combo.add("i'm browsing");
    combo.add("i'm maintain");
    GridData gridData = new GridData();
    gridData.widthHint = 75;
    combo.setLayoutData(gridData);
    return combo; 
  }

  protected Button buildSearchGo(Composite parent) {
    Button b = new Button(parent,SWT.FLAT);
    b.setText("go");
    b.addSelectionListener(new DoSearch());
    return b;  
  }

  //`````````````````````
  // inner classe
  //`````````````````````

  public class DoSearch extends SelectionAdapter {
 
    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent event) {
      doSearch();
    }
  }

}
