package ca.aura.xml;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.AbstractAura;
import ca.aura.artefacts.Aura;
import ca.aura.artefacts.Description;

/**
 * 
 * @author aforward
 *
 */
public class AuraXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_AURA = "aura";


  private static final String ATTRIBUTE_NAME = "name";

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_AURA;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Aura);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {
    
    AbstractAura aura = new AbstractAura("");
    
    if (!canLoad(ele)) {
      return aura;
    }
    
    Attribute attr = ele.getAttribute(ATTRIBUTE_NAME);
    if (attr != null) {
      aura.setName(attr.getValue());  
    }

    DescriptionXmlConverter descrXml = new DescriptionXmlConverter();
    Element descrElement = ele.getChild(descrXml.getElementName());
    if (descrXml.canLoad(descrElement)) {
      aura.setDescription((Description)descrXml.load(descrElement));    
    }

    return aura;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {
    Element ele = new Element(getElementName());
    
    if (!canBuild(obj)) {
      return ele;  
    }

    Aura aura = (Aura)obj;  
    
    ele.setAttribute(ATTRIBUTE_NAME, aura.getName());

    DescriptionXmlConverter descrXml = new DescriptionXmlConverter();
    if (descrXml.canBuild(aura.getDescription())) {
      ele.addContent(descrXml.build(aura.getDescription()));  
    }

    return ele;
  }

}
