package ca.aura.xml;

import java.util.Iterator;

import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.Artefact;
import ca.aura.project.Depot;

/**
 * 
 * @author aforward
 *
 */
public class DepotXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_DEPOT = "depot";
 

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_DEPOT;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    if (ele == null) {
      return false;  
    }
    return ELEMENT_DEPOT.equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Depot);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {
    
    Depot depot = new Depot();
    if (!canLoad(ele)) {
      return depot;  
    }

    ArtefactXmlConverter auraXml = new ArtefactXmlConverter();
    Iterator iter = ele.getChildren(auraXml.getElementName()).iterator();
    while (iter.hasNext()) {
      Element auraElement = (Element)iter.next();
      if (auraXml.canLoad(auraElement)) {
        depot.add((Artefact)auraXml.load(auraElement));     
      }
    }

    return depot;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {
    
    Element ele = new Element(getElementName());
    
    if (!canBuild(obj)) {
      return ele;
    }
    
    Depot depot = (Depot)obj;
    ArtefactXmlConverter axc = new ArtefactXmlConverter();
    Iterator iter = depot.getArtefacts();
    while (iter.hasNext()) {
      Artefact artefact = (Artefact)iter.next();
      if (axc.canBuild(artefact)) {
        ele.addContent(axc.build(artefact));  
      }
    }
    return ele;

  }

}
