package ca.aura.algorithms;

import java.util.Iterator;
import java.util.StringTokenizer;


import ca.aabeka.util.Vector;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.Aura;
import ca.aura.artefacts.Description;

/**
 * 
 * @author aforward
 *
 */
public class TextSearch {

  Vector artefacts_;
  Vector results_;
  
  
  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */

  public TextSearch(Vector artefacts) {
    init(artefacts);  
  }

  /**
   * 
   * give an iterator of artefacts
   * 
   */
  public TextSearch(Iterator iter) {
    Vector v = new Vector();
    v.addAll(iter);
    init(v);  
  }  
  

  private void init(Vector artefacts) {
    artefacts_ = artefacts;
    results_ = new Vector(false);  
  }

  //`````````````````````
  // interface
  //`````````````````````

  /**
   * 
   * search for a particular string
   * 
   */

  public int search(String string) {
    return search(string,true);  
  }


  /**
   * 
   * search for a string, and perhaps reset the results
   * 
   */
  

  public int search(String string, boolean startover) {

    if (startover) {
      results_.removeAllElements();  
    }  
    
    int sizeBefore = results_.size();
    
    StringTokenizer st = new StringTokenizer(string);
    while (st.hasMoreTokens()) {
      String word = st.nextToken();
      results_.addAll(searchForWord(word, artefacts_));  
    }
    
    int sizeAfter = results_.size();
    return sizeAfter - sizeBefore;
  }
  
  public Iterator results() {
    return results_.iterator();  
  }

  public int numberOfResults() {
    return results_.size();    
  }

  //`````````````````````
  // static 
  //`````````````````````

  /**
   * 
   * search for a word in a collection of artefacts
   * 
   */

  protected static Vector searchForWord(String word, Vector artefacts) {
    
    Vector results = new Vector(false);
    
    Iterator iter = artefacts.iterator();
    while (iter.hasNext()) {
      Artefact artefact = (Artefact)iter.next();
      
      Iterator auraIter = artefact.getAura();
      while (auraIter.hasNext()) {
        Aura aura = (Aura)auraIter.next();
        if (contains(aura.getName(),word)) {
          results.add(artefact); 
          break; 
        }

        Description descr = aura.getDescription();
        if (contains(descr.getText(), word)) {
          results.add(artefact);
          break;
        }

        if (contains(descr.getCategory(),word)) {
          results.add(artefact);
          break;  
        }
      }
      
      
    }  
    return results;
  }
  
  /**
   * 
   * does the big text contain the little text?
   * 
   */
  
  protected static boolean contains(String haystack, String needle) {
    if (haystack == null || needle == null) {
      return false;  
    }
    
    int index = haystack.toLowerCase().indexOf(needle.toLowerCase());
    if (index == -1) {
      return false;  
    }

    return true;  
  }  

}
