package ca.aura.ui.swt;


import ca.aura.ui.swt.actions.*;
import ca.aura.ui.swt.actions.LaunchFileAction;
import ca.aabeka.util.Debug;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.Doc;
import ca.aura.artefacts.JavaDoc;
import ca.aura.artefacts.JunitReport;
import ca.aura.artefacts.SourceCode;
import ca.aura.artefacts.Webpage;
import ca.aura.ui.resources.ArtefactResource;


import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Decorations;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

/**
 * 
 * @author Andrew
 *
 */
public class ArtefactView {

  private static final int DEFAULT_X = 10;
  private static final int DEFAULT_Y = 10;

  private Composite parent_;
  private Label self_;

  private Artefact artefact_;

  ImageDescriptor normal_;
  ImageDescriptor highlight_;

  //``````````````````````
  // constructor
  //``````````````````````
  
  /**
   * 
   * default
   * 
   */
  
  protected ArtefactView(Artefact artefact, Composite parent) {
    init(artefact, parent, DEFAULT_X, DEFAULT_Y);  
  }

  /**
   * 
   * specify the location as well
   * 
   */
  
  protected ArtefactView(Artefact artefact, Composite parent, int x, int y) {
    init(artefact, parent, x,y);  
  }

  
  private void init(Artefact artefact, Composite parent, int x, int y) {
    
    artefact_ = artefact;
    
    parent_ = parent;  
    
    self_ = new Label(parent, SWT.SHADOW_IN);
    setImage(artefact);
    
    Image img = normal_.createImage();
    
    self_.setImage(img);
    self_.setBounds(x,y,img.getBounds().width, img.getBounds().height);
    self_.setBackground(parent.getBackground());

    buildMenu();  

    MovingControlListener moving = new MovingControlListener();
    self_.addMouseListener(moving);
    self_.addMouseMoveListener(moving);
    
    self_.setToolTipText(artefact.getName() + ":" + artefact.getDescription().toString());

  }

  //``````````````````````
  // interface
  //``````````````````````

  /**
   * 
   * set the artefact
   * 
   */  

  public Artefact getArtefact() {
    return artefact_;  
  }

  /**
   * 
   * highlight the artefact
   * 
   */
  
  public void highlight() {
    Debug.defaultPrintln(this, "highlighting");
    self_.setImage(highlight_.createImage());      
  }

  /**
   * 
   * normal
   * 
   */
  
  public void normal() {
    self_.setImage(normal_.createImage());  
  }

  /**
   * 
   * build the menu
   * 
   */  

  public Control getControl() {
    return self_;  
  }

  /**
   * 
   * build a  ui
   * 
   */  
 
  public static ArtefactView build(Artefact artefact, Composite parent, int x, int y) {
    ArtefactView ui = new ArtefactView(artefact, parent,x,y);
    return ui;  
  }


  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * add a comment
   * 
   */
  
  public void addComment() {
    Debug.defaultPrintln(this, "adding a comment");
    Decorations decor = new Decorations(parent_,SWT.SHELL_TRIM );
    decor.setBounds(0,0,100,100);
  }
  

  /**
   * 
   * build the menu
   * 
   */    

  protected void buildMenu() {
    Menu menu = new Menu(self_.getShell(), SWT.POP_UP);

    MenuItem browse = new MenuItem(menu, SWT.CASCADE);
    browse.setText(resource().getBrowseMenuItemText());
    browse.addSelectionListener(new ActionSelectionListener(new LaunchFileAction(artefact_.getFile())));

    MenuItem search = new MenuItem(menu, SWT.CASCADE);
    search.setText(resource().getSearchMenuItemText());
    search.setEnabled(false);

    MenuItem addComment = new MenuItem(menu, SWT.CASCADE);
    addComment.setText(resource().getAddCommentMenuItemText());
    addComment.addSelectionListener(new AddComment());
    addComment.setEnabled(false);

    self_.setMenu(menu);
  }
  
  /**
   * 
   * get the ui resource
   * 
   */  

  protected ArtefactResource resource() {
    return ArtefactResource.getDefault();  
  }

  
  /**
   * 
   * get teh image
   * 
   */
  
  protected void setImage(Artefact artefact) {
    Debug.defaultPrintln(this,"get the image");
    
    if (artefact instanceof Doc) {
      normal_ = resource().getDocImageDescriptor();
      highlight_ = resource().getDocHighlightImageDescriptor();
      return;
    } 
    
    if (artefact instanceof Webpage) {
      normal_ = resource().getWebpageImageDescriptor();
      highlight_ = resource().getWebpageHighlightImageDescriptor();
      return;
    } 
    
    if (artefact instanceof SourceCode) {
      normal_ = resource().getSourceCodeImageDescriptor();
      highlight_ = resource().getSourceCodeHighlightImageDescriptor();
      return;
    }

    if (artefact instanceof JavaDoc) {
      normal_ = resource().getJavaDocImageDescriptor();
      highlight_ = resource().getJavaDocHighlightImageDescriptor();
      return;
    }

    if (artefact instanceof JunitReport) {
      normal_ = resource().getJunitReportImageDescriptor();
      highlight_ = resource().getJunitReportHighlightImageDescriptor();
      return;
    }
    
    
    
      normal_ = resource().getDocImageDescriptor();
      highlight_ = resource().getDocHighlightImageDescriptor();
  }

  //`````````````````````
  // inner classe
  //`````````````````````

  public class AddComment extends SelectionAdapter {
 
    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent event) {
      addComment();
    }
  }

  public class LaunchDocument extends SelectionAdapter {
 
    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent event) {
      addComment();
    }
  }



}
