package ca.aura.ui.swt;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import ca.aabeka.util.Debug;
import ca.aabeka.util.Hashtable;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.Aura;
import ca.aura.artefacts.Doc;
import ca.aura.artefacts.SourceCode;
import ca.aura.artefacts.Webpage;
import ca.aura.artefacts.sorting.Data;
import ca.aura.artefacts.sorting.Folder;
import ca.aura.ui.resources.ArtefactResource;

/**
 * 
 * @author aforward
 *
 */
public class StructureLabelProvider extends LabelProvider {

  Hashtable hash_;
  
  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  
  public StructureLabelProvider() {
    hash_ = new Hashtable();
  }


  //`````````````````````
  // ILableProvider interface
  //`````````````````````

  /**
   * 
   * @see org.eclipse.jface.viewers.ILabelProvider#getImage(Object)
   * 
   */
  public Image getImage(Object obj) {
    ImageDescriptor idescr = getImageDescriptor(obj);
    
    if (hash_.containsKey(idescr)) {
      return (Image)hash_.get(idescr);  
    }
    
    Image image = idescr.createImage();
    hash_.put(idescr, image);
    return image;
  }

  /**
   * 
   * @see org.eclipse.jface.viewers.ILabelProvider#getText(Object)
   * 
   */
  public String getText(Object obj) {
    
    if (obj instanceof Data) {
      Data data = (Data)obj;
      if (data.hasData() && data.getData().numberOfAura() == 1) {
        return data.getData().getName();  
      }
    }
    
    return obj.toString();
  }

  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * get the image descriptor
   * 
   */

  public ImageDescriptor getImageDescriptor(Object obj) {

    Debug.defaultPrintln(this,"inside aura");
    
    if (obj instanceof Folder) {
      return ArtefactResource.getDefault().getFolderImage();  
    }

    if (obj instanceof Data) {
      Artefact artefact = ((Data)obj).getData();

      if (artefact == null) {
        return getUnknownElementDescriptor();  
      }

      if (artefact instanceof Doc) {
        return ArtefactResource.getDefault().getDocImageDescriptor();    
      }

      if (artefact instanceof Webpage) {
        return ArtefactResource.getDefault().getWebpageImageDescriptor();    
      }
      
      if (artefact instanceof SourceCode) {
        return ArtefactResource.getDefault().getSourceCodeImageDescriptor();    
      }
      
      return getUnknownElementDescriptor();
    }

    if (obj instanceof Aura) {
      Debug.defaultPrintln(this,"inside aura");
      return ArtefactResource.getDefault().getAuraImageDescriptor();  
    }    
    

    return getUnknownElementDescriptor();
  }

  /**
   * @see ca.aabeka.ui.NodeLabelProvider#getUnknownElementDescriptor()
   */
  public ImageDescriptor getUnknownElementDescriptor() {
    return ImageDescriptor.getMissingImageDescriptor();
  }

}
