package ca.aura.ui.swt.actions;

import java.util.Vector;

import ca.aabeka.ui.IPresenter;

/**
 * @author Andrew
 *
 */
public abstract class AbstractAction implements Action {

  private Object obj_;
  private IPresenter controller_;
  
  private Vector history_;
  private int currentIndex_;   
  private int maxIndex_;

  //```````````````````
  // constructor
  //```````````````````

  /**
   * 
   * default
   * 
   */

  public AbstractAction(Object obj, IPresenter controller) {
    init(obj, controller);  
  }  

  private void init(Object obj, IPresenter controller) {
    obj_ = obj;
    controller_ = controller;  
    history_ = new Vector();
    currentIndex_ = -1;  
    maxIndex_ = -1;
  }

  //```````````````````
  // abstract
  //```````````````````

  /**
   * 
   * copy the object
   * 
   */

  public abstract Object copyObject(Object o);

  //```````````````````
  // Action interface
  //```````````````````
  

  /**
   * 
   * do it
   * 
   */
   
  
  public boolean execute() {
    history_.add(++currentIndex_,controller_.getObject());
    maxIndex_ = currentIndex_;
    return setCurrentObject();    
  }

  /**
   * 
   * redo it
   * 
   */

  public boolean redoo() {
    int nextIndex = currentIndex_ + 1;
    return setCurrentObject(nextIndex);
  }

  /**
   * 
   * undo it
   * 
   */

  public boolean undoo() {
    int lastIndex = currentIndex_ - 1;
    return setCurrentObject(lastIndex);
  }

  //```````````````````
  // protected
  //```````````````````

  /**
   * 
   * set the current object
   * 
   */  

  protected boolean setCurrentObject() {
    return setCurrentObject(currentIndex_);  
  }

  /**
   * 
   * set the object
   * 
   */  

  protected boolean setCurrentObject(int index) {
    if (!isIndexOkay(index)) {
      return false;    
    }
    
    currentIndex_ = index;
    obj_ = copyObject(history_.elementAt(currentIndex_));          
    controller_.setObject(obj_);
    return true;
  }

  /**
   * 
   * is the index okay?
   * 
   */

  protected boolean isIndexOkay(int index) {
    return index  >= 0 && index <= maxIndex_;
  }

}
