package ca.aura.ui.swt.actions;

import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;


public class CaretListener implements MouseListener, MouseMoveListener {

  private boolean isSelected_;
  private Point lastPoint_;


  //````````````````````````
  // Constructor
  //````````````````````````

  /**
   *  
   * Specify the presenter to select.
   * 
   */

  public CaretListener(Point lastPoint) {
    init(lastPoint);  
  }

  private void init(Point lastPoint) {
    lastPoint_ = lastPoint;
  }



  //````````````````````````
  // MouseListener
  //````````````````````````

  /**
   *  
   * capture the last point when the mouse is down
   * 
   */

  public void mouseDown(MouseEvent e) {
    isSelected_ = true;
    setPoint(e);
  }

  /**
   *  
   * The document is opened when double clicked
   * 
   */

  public void mouseDoubleClick(MouseEvent e) { 
    isSelected_ = false;
    setPoint(e);
  }
  
  public void mouseUp(MouseEvent e) {
    isSelected_ = false;
    setPoint(e);
  }

  


  //``````````````
  // MouseMoveListener
  //``````````````

  /**
   *  
   * did the mouse move?
   * 
   */
  
  public void mouseMove(MouseEvent e) {

    if (isSelected()) {
      setPoint(e);
    }

  }


  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * set the last point
   * 
   */
  
  protected void setPoint(MouseEvent e) {
    lastPoint_.x = e.x;
    lastPoint_.y = e.y;
  }

  /**
   *  
   * Is the document currently selected?
   * 
   */
  
  protected boolean isSelected() {
    return isSelected_;  
  }

  /**
   *  
   * Set the document as selected or not
   * 
   */

  protected void setSelected(boolean isSelected) {
    isSelected_ = isSelected;  
  }

  
}
