package ca.aura.xml;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.Description;

/**
 * 
 * @author aforward
 *
 */
public class DescriptionXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_DESCRIPTION = "description";
  private static final String ATTRIBUTE_CATEGORY = "category";    

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_DESCRIPTION;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Description);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {
    
    Description descr = new Description();
    
    if (!canLoad(ele)) {
      return descr;  
    }
    
    Attribute attr = ele.getAttribute(ATTRIBUTE_CATEGORY);
    if (attr != null) {
      descr.setCategory(attr.getValue()); 
    }

    descr.setText(ele.getTextTrim());

    return descr;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {

    Element ele = new Element(getElementName());
    
    if (!canBuild(obj)) {
      return ele;  
    }

    Description descr = (Description)obj;
    
    ele.setAttribute(ATTRIBUTE_CATEGORY, descr.getCategory());
    ele.setText(descr.getText());

    return ele;
  }

}
