package ca.aura.xml;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.util.Debug;
import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.project.DefaultProject;
import ca.aura.project.Project;

/**
 * 
 * @author aforward
 *
 */
public class ProjectXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_PROJECT = "project";
  private static final String ELEMENT_DEPOT = "depot";
  private static final String ELEMENT_STRUCTURE = "structure";
  private static final String ELEMENT_RELATION = "association";
  private static final String ELEMENT_HISTORY = "history";  
  
  private static final String ATTRIBUTE_NAME = "name";

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_PROJECT;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Project);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {

    DefaultProject obj = new DefaultProject();

    if (!canLoad(ele)) {
      return obj;  
    }

    Attribute attr = ele.getAttribute(ATTRIBUTE_NAME);
    
    if (attr != null) {
       Debug.defaultPrintln(this,"setting the project name:" + attr.getValue());
       obj.setName(attr.getValue());
       Debug.defaultPrintln(this,"setting the project name:" + obj);
    }
    
    Element child;
    
    child = ele.getChild(ELEMENT_DEPOT);
    if (child != null) {
      obj.setDepotFile(child.getTextTrim());
    }

    
    child = ele.getChild(ELEMENT_STRUCTURE);
    if (child != null) {
      obj.setStructureFile(child.getTextTrim());  
    }    

    child = ele.getChild(this.ELEMENT_RELATION);
    if (child != null) {
      obj.setRelationFile(child.getTextTrim());  
    }    

    child = ele.getChild(ELEMENT_HISTORY);
    if (child != null) {
      obj.setHistoryFile(child.getTextTrim());
    }


    return obj;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {
    
    Element ele = new Element(ELEMENT_PROJECT);
    
    if (!canBuild(obj)) {
      return ele;  
    }
    
    Project project = (Project)obj;
    ele.setAttribute(ATTRIBUTE_NAME,project.getName());
    
    Element child;
    
    child = new Element(ELEMENT_DEPOT);
    child.setText(project.getDepotFile());
    ele.addContent(child);
    
    child = new Element(ELEMENT_STRUCTURE);
    child.setText(project.getStructureFile());
    ele.addContent(child);
    
    child = new Element(ELEMENT_RELATION);    
    child.setText(project.getRelationFile());
    ele.addContent(child);

    child = new Element(ELEMENT_HISTORY);    
    child.setText(project.getHistoryFile());
    ele.addContent(child);


    return ele;
  }

}
