package ca.aura.xml;

import java.util.Iterator;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefact.Artefact;

/**
 * 
 * @author Andrew
 *
 */
public abstract class ArtefactXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_ARTEFACT = "artefact";
  private static final String ELEMENT_NAME = "name";
  private static final String ELEMENT_REFERENCES = "references";

  private static final String ATTRIBUTE_CATEGORY = "category";
  private static final String ATTRIBUTE_ID = "id";

  public static final String ELEMENT_ROOT = ELEMENT_ARTEFACT;  

  //``````````````````````
  // abstract interface
  //``````````````````````

  /**
   * 
   * get the category value
   * 
   */
  
  public abstract String getCategoryValue();


  //``````````````````````
  // interface
  //``````````````````````

  /**
   * 
   * get the references
   * 
   */
  
  public Vector getReferences(Element artefact) {
    
    Vector references = new Vector();
    
    if (artefact == null) {
      return references;  
    }  
    
    Element referencesElement = artefact.getChild(ELEMENT_REFERENCES);
    if (referencesElement == null) {
      return references;	
    }
    
    Iterator iter = referencesElement.getChildren().iterator();    
    while (iter.hasNext()) {
      Element ref = (Element) iter.next();
      if (ReferenceXmlConverter.ELEMENT_ROOT.equals(ref.getName())) {
        ReferenceXmlConverter referenceConverter = new ReferenceXmlConverter();
        references.addElement(referenceConverter.load(ref));  
      }
    }
    
    return references;    
  }


  //``````````````````````
  // ObjectXmlConverter interface
  //``````````````````````

  /**
   * 
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   * 
   * get the Element name
   * 
   */

  public String getElementName() {
    return ELEMENT_ROOT;  
  }

  /**
   * 
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   * 
   * 
   * 
   * <artefact category="document" id="123" >
   *   <name>The Documents Name</name>
   *   <description>A blurb about the document
   *     <type>High Level Document</type>
   *   </description>
   *   <location>file://blahblah</location>
   * 
   *   '' not available yet ''
   *   <content>
   *     <artefact category="section" id="453">
   *       <name>Description</name>
   *       <description>blah blah</desription>
   *       <type>Graph</type>
   *     </artefact>
   *   </content>
   *   <references>
   *     <artefact id="321" />
   *   </references>
   *   '' not available yet ''
   * 
   * </artefact>
   * 
   * 
   * 
   * 
   * 
   */
  public abstract Object load(Element ele);

  /**
   * 
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   * 
   */
  public Element build(Object obj) {
    if (!(obj instanceof Artefact)) {
      return new Element(ELEMENT_ARTEFACT); 
    }
    
    return buildArtefact((Artefact)obj);
  }

  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * build an element from a artefact for real :-)
   * 
   */
  
  protected Element buildArtefact(Artefact artefact) {
    
    Element root = new Element(ELEMENT_ARTEFACT);
    Attribute category = new Attribute(ATTRIBUTE_CATEGORY, getCategoryValue());
    Attribute id = new Attribute(ATTRIBUTE_ID, artefact.getId());
    
    root.setAttribute(category);
    root.setAttribute(id);

    Element name = new Element(ELEMENT_NAME);
    name.setText(artefact.getName());

    ObjectXmlConverter descrXml = new DescriptionXmlConverter();    
    root.addContent(descrXml.build(artefact.getDescription()));

    ObjectXmlConverter locationXml = new LocationXmlConverter();
    root.addContent(locationXml.build(artefact.getLocation()));

    
    return root;
  }



  /**
   * 
   * build the references
   * 
   * 
   */

  public Element buildReferences(Artefact artefact) {
    Element references = new Element(ELEMENT_REFERENCES);
    
    Iterator referencesIter = artefact.getReferences();
    while (referencesIter.hasNext()) {
      ReferenceXmlConverter referenceConverter = new ReferenceXmlConverter();
      references.addContent(referenceConverter.build(referencesIter.next()));
    }
    
    return references;    
  }

}
