package ca.aura.xml;

import java.util.Iterator;
import java.util.Vector;

import org.jdom.Element;

import ca.aura.artefact.Doc;
import ca.aura.artefact.components.Description;
import ca.aura.artefact.components.Location;

/**
 * 
 * @author Andrew
 *
 */
public class DocXmlConverter extends ArtefactXmlConverter{

  protected static final String ELEMENT_CONTENT = "content";


  private static final String VALUE_CATEGORY = "document";


  /**
   * 
   * @see ca.aura.xml.ArtefactXmlConverter#getCategoryValue()
   * 
   */
  public String getCategoryValue() {
    return VALUE_CATEGORY;
  }



  //``````````````````````
  // ArtefactXmlConverter interface
  //``````````````````````

  /**
   * 
   * add the documents content when building the Element
   * 
   */
  
  public Element build(Object obj) {
    Element root = super.build(obj);
    
    if (!(obj instanceof Doc)) {
      return root;	
    }
    
    Doc doc = (Doc) obj;
    root.addContent(buildContent(doc));
    return root;
      
  }


  /**
   * 
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   * 
   *   a document can also contain section artefacts
   * 
   *   <content>
   *     <artefact category="section" id="453">
   *       <name>Description</name>
   *       <description>blah blah</desription>
   *       <type>Graph</type>
   *     </artefact>
   *   </content>
   * 
   */
  public Object load(Element ele) {

    String id = getId(ele);
    String name = getName(ele);
    Location location = getLocation(ele);
    Description descr = getDescription(ele);
    Vector references = getReferences(ele);
    Vector content = loadContent(ele);
    

    Doc doc = new Doc(id,name);
    doc.setDesription(descr);
    doc.setLocation(location);
    doc.setReferences(references);
    doc.setContent(content);

    return doc; 
  }


  //``````````````````````
  // protected
  //``````````````````````

  /**
   * 
   * build an content 
   * 
   * 
   */

  public Element buildContent(Doc doc) {
    
    Element content = new Element(ELEMENT_CONTENT);
    
    Iterator contentIter = doc.getContent();
    while(contentIter.hasNext()) {
      FragmentXmlConverter sectionConverter = new FragmentXmlConverter();
      content.addContent(sectionConverter.build(contentIter.next()));
    }
    
    return content;
  }


  /**
   * 
   * load the content 
   * 
   * 
   */

  public Vector loadContent(Element ele) {
    
    Vector content = new Vector();
    
    if (ele == null) {
      return content;  
    }
    
    if (!ELEMENT_CONTENT.equals(ele.getName())) {
      return  content;
    }

    Element contentElement = ele.getChild(ELEMENT_CONTENT);
    if (contentElement == null) {
      return content;  
    }
    
    Iterator iter = contentElement.getChildren().iterator();    
    while (iter.hasNext()) {
      Element section = (Element) iter.next();
      if (FragmentXmlConverter.ELEMENT_ROOT.equals(section.getName())) {
        FragmentXmlConverter sectionConverter = new FragmentXmlConverter();
        content.addElement(sectionConverter.load(section));  
      }
    }
    
    return content;  
    
  }

}
