package ca.aura.algorithms;

import ca.aabeka.util.DateAndTime;
import ca.aabeka.util.DefaultGraph;
import ca.aabeka.util.Graph;
import ca.aabeka.util.Hashtable;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.reports.DefaultReport;
import ca.aura.artefacts.reports.Report;
import ca.aura.artefacts.usage.DefaultUsage;
import ca.aura.artefacts.usage.Usage;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * 
 * @author aforward
 *
 */
public class ReferentialIntegrityTest extends TestCase {

  //```````````````````
  // all tests
  //```````````````````

  /**
   * 
   * the main runner
   * 
   */

  public static void main() {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * 
   * the test suite to group tests together
   * 
   */

  public static Test suite() {
    return new TestSuite(ReferentialIntegrityTest.class);
  }


  //`````````````````````
  // setup
  //`````````````````````

  /**
   * 
   * create a tmp artefact
   * 
   */

  public class TmpArtefact extends AbstractArtefact {
  
    public TmpArtefact(int id) {
      super(id + "");
    }   
  }
  

  //`````````````````````
  // assert getReferentialIntegrity
  //`````````````````````

  /**
   * 
   * assert getReferentialIntegrities
   * 
   */
  
  public void assert_getReferentialIntegrities(Report report, Graph g, Hashtable expected) {
    ReferentialIntegrity ri = do_ri(report, g,1);
    Hashtable actual = ri.getResults();
    assertEquals(expected,actual);    
  }

  /**
   * 
   * do the RI calculation
   * 
   */
  
  public ReferentialIntegrity do_ri(Report report, Graph g, int k) {
    ReferentialIntegrity ri = new  ReferentialIntegrity(report, g,k);  
    return ri;
  }


  //`````````````````````
  // tests
  //`````````````````````

  /**
   * 
   * test the algorithm
   * 
   */
  
  public void test_algorithm() {

    DefaultReport report = new DefaultReport(DateAndTime.add(0));
    
    Usage usage;
    
    
    Artefact a1 = new TmpArtefact(1);
    usage = new DefaultUsage(a1.getId(),DateAndTime.add(-3));
    report.add(a1, usage);

    Artefact a2 = new TmpArtefact(2);
    usage = new DefaultUsage(a2.getId(),DateAndTime.add(-1));
    report.add(a2, usage);

    Artefact a3 = new TmpArtefact(3);    
    usage = new DefaultUsage(a3.getId(),DateAndTime.add(-9));
    report.add(a3, usage);    

    Artefact a4 = new TmpArtefact(4);    
    usage = new DefaultUsage(a4.getId(),DateAndTime.add(-8));
    report.add(a4, usage);    

    
    Graph g = new DefaultGraph();
    g.add(a1, a3);
    g.add(a2, a1);
    g.add(a2, a3);
    g.add(a3, a4);    
   
    //double zShift = -1.2925912266482087;
    
    Hashtable expected = new Hashtable();
    expected.put(a1, new Double(-0.08617274844321392));
    expected.put(a2, new Double(0.25851824532964174));
    expected.put(a3, new Double(1.120245729761781));
    expected.put(a4, new Double(-1.2925912266482087));    

    assert_getReferentialIntegrities(report, g, expected);
    do_ri(report,g,20);
  }


}
