package ca.aura.algorithms;

import ca.aabeka.util.Vector;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.Description;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * 
 * @author aforward
 *
 */
public class TextSearchTest extends TestCase {


  //```````````````````
  // all tests
  //```````````````````

  /**
   * 
   * the main runner
   * 
   */

  public static void main() {
    junit.textui.TestRunner.run(suite());
  }

  /**
   * 
   * the test suite to group tests together
   * 
   */

  public static Test suite() {
    return new TestSuite(TextSearchTest.class);
  }

  //`````````````````````
  // setUp
  //`````````````````````

  /**
   * 
   * create a Tmp Artefact
   * 
   */

  public class Tmp extends AbstractArtefact {
  
    public Tmp(String id, String name, String descr, String type) {
      super(id);
      super.setName(name);
      super.setDescription(new Description(type,descr));
    }  
    
    
  }

  //`````````````````````
  // assertions
  //`````````````````````

  /**
   * 
   * assert contains
   * 
   */
  
  public void assert_contains(String bigword, String littleword, boolean expected) {
    boolean actual = TextSearch.contains(bigword, littleword);
    assertEquals(expected, actual);
  }
  
  
  /**
   * 
   * assert searchForWord
   * 
   */
  
  public void assert_searchForWord(String word, Vector artefacts, Vector expected) {
    Vector actual = TextSearch.searchForWord(word, artefacts);
    assertEquals(expected,actual);
  }


  /**
   * 
   * assert search
   * 
   */
  public void assert_search(String words, Vector artefacts, Vector expected, int expectedResultsFound) {
    TextSearch ts = new TextSearch(artefacts);
    int actualResultsFound = ts.search(words);
    assertEquals(expectedResultsFound, actualResultsFound);
    Vector actual = new Vector();
    actual.addAll(ts.results());
    assertEquals(expected, actual); 
    
  }
  
  /**
   * 
   * assert_searchAppend
   * 
   */
  
  public void assert_searchAppend(String words, Vector artefacts, Vector expected, int expectedResultsFound) {
    TextSearch ts = new TextSearch(artefacts);
    int actualResultsFound = ts.search(words, true);
    assertEquals(expectedResultsFound, actualResultsFound);
    Vector actual = new Vector();
    actual.addAll(ts.results());
    assertEquals(expected, actual); 
    
  }
  
  
  //`````````````````````
  // tests
  //`````````````````````

  /**
   * 
   * contains
   * 
   */
  
  public void test_contains_null() {
    assert_contains(null,null,false);  
    assert_contains("asdf",null,false);  
    assert_contains(null,"asdf",false);      
  }
  
  public void test_contains_subPartOfTheWord() {
    assert_contains("afeAbDerw","abD", true);  
  }

  public void test_contains() {
    assert_contains("afe AbD erw","abD", true);  
  }


  /**
   * 
   * searchForWord
   * 
   */

  public void test_searchForWord() {
    
    String word = "one";
    
    Artefact a1 = new Tmp("1","oNe","slkfjsa","lwef lsafjs");
    Artefact a2 = new Tmp("2","oN","slkonefjsa","lwef lsafjs");
    Artefact a3 = new Tmp("3","oN","one slk  fjsa","lwef lsafjs");
    Artefact a4 = new Tmp("3","oN","slk one fjsa","lwef lsfjs one");   
   
    Vector expected = new Vector();
    expected.add(a1);
    expected.add(a3);
    expected.add(a4);    

    Vector artefacts = new Vector();
    artefacts.add(a1);
    artefacts.add(a2);
    artefacts.add(a3);  
    artefacts.add(a4);    

    assert_searchForWord(word,artefacts,expected);

  }

  /**
   * 
   * searchForWord
   * 
   */

  public void test_search() {
    
    String words = "one two three" ;
    
    Artefact a1 = new Tmp("1","oNe","123 34 432","43 23");
    Artefact a2 = new Tmp("2","11","12 1two32 32","12 3 134123");
    Artefact a3 = new Tmp("3","1","123 4321","1234 43 3234");
    Artefact a4 = new Tmp("4","33","323 534","1234 three");  

    Vector expected = new Vector();
    expected.add(a1);
    expected.add(a2);
    expected.add(a4);    

    Vector artefacts = new Vector();
    artefacts.add(a1);
    artefacts.add(a2);
    artefacts.add(a3);  
    artefacts.add(a4);  
    
    assert_search(words, artefacts, expected, expected.size());

  }

  public void test_search_addToResults() {
    
    
    
    Artefact a1 = new Tmp("1","oNe","123 34 432","43 23");
    Artefact a2 = new Tmp("2","11","12 1two32 32","12 3 134123");
    Artefact a3 = new Tmp("3","1","123 4321","1234 43 3234");
    Artefact a4 = new Tmp("4","33","323 534","1234 three");   
    Artefact a5 = new Tmp("5","12five","12 314 ","3432 4234");  

    Vector artefacts = new Vector();
    artefacts.add(a1);
    artefacts.add(a2);
    artefacts.add(a3);  
    artefacts.add(a4);  
    artefacts.add(a5);

    String words;
    Vector expected;
    

    words = "five";
    expected = new Vector();
    expected.add(a5);
    assert_searchAppend(words, artefacts, expected, 1);

    words = "one two three" ;
    expected = new Vector();
    expected.add(a1);
    expected.add(a2);
    expected.add(a4); 
    
    assert_searchAppend(words, artefacts, expected, 3);

  }


}
