package ca.aura.algorithms.normalizer;

import java.util.Vector;

import ca.aabeka.math.Sequence;
import ca.aura.algorithms.rank.Rank;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @author aforward
 *
 */
public abstract class AbstractRankNormalizerTest extends TestCase {

  //````````````````````
  // for all tests
  //````````````````````  
  
  public static void main () {
    junit.textui.TestRunner.run (suite());
  }

  public static Test suite() {
    return new TestSuite(AbstractRankNormalizerTest.class);
  }  

  //````````````````````
  // setup / teardown
  //```````````````````` 

  protected static Sequence buildSequence() {
  	Vector numbers = new Vector();
  	numbers.add("3");
  	numbers.add("7");
  	numbers.add("2");
  	numbers.add("5");  	
  	numbers.add("3");
  	
    Sequence seq = new Sequence(numbers);
    return seq;
  }


  //````````````````````
  // assertions
  //```````````````````` 

  /**
   * 
   * assert getRank
   * 
   */

  public void assert_getRank(RankNormalizer norm, int rank) {
    double value = getCoef(rank);
    assert_getRank(norm,rank,value);
  }


  public void assert_getRank(RankNormalizer norm, int rank, double value) {
    Rank expected = new Rank(rank);  
    Rank actual = norm.getRank(value);
    assertEquals(expected,actual);  
  }


  //````````````````````
  // abstract
  //```````````````````` 

  public abstract RankNormalizer getRankNormalizer(Sequence sequence);
  public abstract double getCoef(int rank);




  //````````````````````
  // test cases
  //```````````````````` 

  /**
   * 
   * abstract test_cases
   * 
   */

  public void test_getRank_invalidDistribution() {
    assert_getRank(getRankNormalizer(new Sequence()),Rank.getUnknown());
  }

  public void test_getRank_1() {
    assert_getRank(getRankNormalizer(buildSequence()),1);
  }

  public void test_getRank_2() {
    assert_getRank(getRankNormalizer(buildSequence()),2);
  }

  public void test_getRank_3() {
    assert_getRank(getRankNormalizer(buildSequence()),3);
  }

  public void test_getRank_4() {
    assert_getRank(getRankNormalizer(buildSequence()),4);    
  }
  

  
}
